#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests for the BagMRU Windows Registry plugin."""

from __future__ import unicode_literals

import unittest

from plaso.formatters import winreg  # pylint: disable=unused-import
from plaso.parsers.winreg_plugins import bagmru

from tests import test_lib as shared_test_lib
from tests.parsers.winreg_plugins import test_lib


class TestBagMRUPlugin(test_lib.RegistryPluginTestCase):
  """Tests for the BagMRU plugin."""

  def testFilters(self):
    """Tests the FILTERS class attribute."""
    plugin = bagmru.BagMRUPlugin()

    key_path = (
        'HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\Shell\\BagMRU')
    self._AssertFiltersOnKeyPath(plugin, key_path)

    key_path = (
        'HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\ShellNoRoam\\'
        'BagMRU')
    self._AssertFiltersOnKeyPath(plugin, key_path)

    key_path = (
        'HKEY_CURRENT_USER\\Software\\Classes\\Local Settings\\Software\\'
        'Microsoft\\Windows\\Shell\\BagMRU')
    self._AssertFiltersOnKeyPath(plugin, key_path)

    key_path = (
        'HKEY_CURRENT_USER\\Software\\Classes\\Local Settings\\Software\\'
        'Microsoft\\Windows\\ShellNoRoam\\BagMRU')
    self._AssertFiltersOnKeyPath(plugin, key_path)

    key_path = (
        'HKEY_CURRENT_USER\\Local Settings\\Software\\Microsoft\\Windows\\'
        'Shell\\BagMRU')
    self._AssertFiltersOnKeyPath(plugin, key_path)

    key_path = (
        'HKEY_CURRENT_USER\\Local Settings\\Software\\Microsoft\\Windows\\'
        'ShellNoRoam\\BagMRU')
    self._AssertFiltersOnKeyPath(plugin, key_path)

    self._AssertNotFiltersOnKeyPath(plugin, 'HKEY_LOCAL_MACHINE\\Bogus')

  @shared_test_lib.skipUnlessHasTestFile(['NTUSER.DAT'])
  def testProcess(self):
    """Tests the Process function."""
    plugin = bagmru.BagMRUPlugin()
    test_file_entry = self._GetTestFileEntry(['NTUSER.DAT'])
    key_path = (
        'HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\ShellNoRoam\\BagMRU')

    win_registry = self._GetWinRegistryFromFileEntry(test_file_entry)
    registry_key = win_registry.GetKeyByPath(key_path)
    storage_writer = self._ParseKeyWithPlugin(
        registry_key, plugin, file_entry=test_file_entry)

    self.assertEqual(storage_writer.number_of_events, 15)

    events = list(storage_writer.GetEvents())

    event = events[0]

    self.assertEqual(event.pathspec, test_file_entry.path_spec)
    # This should just be the plugin name, as we're invoking it directly,
    # and not through the parser.
    self.assertEqual(event.parser, plugin.plugin_name)

    self.CheckTimestamp(event.timestamp, '2009-08-04 15:19:16.997750')

    expected_message = (
        '[{0:s}] '
        'Index: 1 [MRU Value 0]: '
        'Shell item path: <My Computer>').format(key_path)
    expected_short_message = '{0:s}...'.format(expected_message[:77])

    self._TestGetMessageStrings(event, expected_message, expected_short_message)

    event = events[1]

    self.CheckTimestamp(event.timestamp, '2009-08-04 15:19:10.669625')

    expected_message = (
        '[{0:s}\\0] '
        'Index: 1 [MRU Value 0]: '
        'Shell item path: <My Computer> C:\\').format(key_path)
    expected_short_message = '{0:s}...'.format(expected_message[:77])

    self._TestGetMessageStrings(event, expected_message, expected_short_message)

    event = events[14]

    self.CheckTimestamp(event.timestamp, '2009-08-04 15:19:16.997750')

    # The winreg_formatter will add a space after the key path even when there
    # is not text.
    expected_message = '[{0:s}\\0\\0\\0\\0\\0] '.format(key_path)

    self._TestGetMessageStrings(event, expected_message, expected_message)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests for the psteal CLI tool."""

from __future__ import unicode_literals

import os
import unittest

from dfvfs.resolver import resolver as dfvfs_resolver

from plaso.cli import psteal_tool
from plaso.lib import errors

from tests import test_lib as shared_test_lib
from tests.cli import test_lib


class PstealToolTest(test_lib.CLIToolTestCase):
  """Tests for the psteal CLI tool."""

  # pylint: disable=protected-access

  _BDE_PASSWORD = 'bde-TEST'

  _EXPECTED_PROCESSING_OPTIONS = '\n'.join([
      'usage: psteal_test.py',
      '',
      'Test argument parser.',
      ''])

  _STORAGE_FILENAME_TEMPLATE = r'\d{{8}}T\d{{6}}-{filename}.plaso'

  def _CheckOutput(self, output, expected_output):
    """Compares the output against the expected output.

    The actual processing time is ignored, since it can vary.

    Args:
      output (bytes): tool output.
      expected_output (list[bytes]): expected tool output.
    """
    output = output.split(b'\n')

    self.assertEqual(output[:3], expected_output[:3])
    self.assertTrue(output[3].startswith(b'Processing time\t: '))
    self.assertEqual(output[4:], expected_output[4:])

  def testGenerateStorageFileName(self):
    """Tests the _GenerateStorageFileName function."""
    test_tool = psteal_tool.PstealTool()

    test_tool._source_path = '/test/storage/path'
    storage_filename = test_tool._GenerateStorageFileName()
    expected_storage_filename = self._STORAGE_FILENAME_TEMPLATE.format(
        filename='path')
    # pylint: disable=deprecated-method
    self.assertRegexpMatches(storage_filename, expected_storage_filename)

    test_tool._source_path = '/test/storage/path/'
    storage_filename = test_tool._GenerateStorageFileName()
    expected_storage_filename = self._STORAGE_FILENAME_TEMPLATE.format(
        filename='path')
    # pylint: disable=deprecated-method
    self.assertRegexpMatches(storage_filename, expected_storage_filename)

    test_tool._source_path = '/'
    storage_filename = test_tool._GenerateStorageFileName()
    expected_storage_filename = self._STORAGE_FILENAME_TEMPLATE.format(
        filename='ROOT')
    # pylint: disable=deprecated-method
    self.assertRegexpMatches(storage_filename, expected_storage_filename)

    test_tool._source_path = '/foo/..'
    storage_filename = test_tool._GenerateStorageFileName()
    expected_storage_filename = self._STORAGE_FILENAME_TEMPLATE.format(
        filename='ROOT')
    # pylint: disable=deprecated-method
    self.assertRegexpMatches(storage_filename, expected_storage_filename)

    test_tool._source_path = 'foo/../bar'
    storage_filename = test_tool._GenerateStorageFileName()
    expected_storage_filename = self._STORAGE_FILENAME_TEMPLATE.format(
        filename='bar')
    # pylint: disable=deprecated-method
    self.assertRegexpMatches(storage_filename, expected_storage_filename)

  @shared_test_lib.skipUnlessHasTestFile(['artifacts'])
  @shared_test_lib.skipUnlessHasTestFile(['psort_test.plaso'])
  def testFailWhenOutputAlreadyExists(self):
    """Test to make sure the tool raises when the output file already exists."""
    output_writer = test_lib.TestOutputWriter(encoding='utf-8')
    test_tool = psteal_tool.PstealTool(output_writer=output_writer)

    options = test_lib.TestOptions()
    options.artifact_definitions_path = self._GetTestFilePath(['artifacts'])
    options.storage_file = self._GetTestFilePath(['psort_test.plaso'])
    options.source = 'unused_source'

    with shared_test_lib.TempDirectory() as temp_directory:
      options.write = os.path.join(temp_directory, 'output.txt')

      with open(options.write, 'w') as file_object:
        file_object.write('bogus')

      # Test when output file already exists.
      # Escape \ otherwise assertRaisesRegexp can error with:
      # error: bogus escape: '\\1'
      expected_error = 'Output file already exists: {0:s}.'.format(
          options.write.replace('\\', '\\\\'))
      # pylint: disable=deprecated-method
      with self.assertRaisesRegexp(errors.BadConfigOption, expected_error):
        test_tool.ParseOptions(options)

  @shared_test_lib.skipUnlessHasTestFile(['testdir'])
  def testParseOptions(self):
    """Tests the ParseOptions function."""
    output_writer = test_lib.TestOutputWriter(encoding='utf-8')
    test_tool = psteal_tool.PstealTool(output_writer=output_writer)

    options = test_lib.TestOptions()
    options.source = 'source'
    # Test when the output file is missing.
    expected_error = 'Output format: dynamic requires an output file'
    # pylint: disable=deprecated-method
    with self.assertRaisesRegexp(errors.BadConfigOption, expected_error):
      test_tool.ParseOptions(options)

    options = test_lib.TestOptions()
    options.write = 'output.csv'
    # Test when the source is missing.
    expected_error = 'Missing source path.'

    # pylint: disable=deprecated-method
    with self.assertRaisesRegexp(errors.BadConfigOption, expected_error):
      test_tool.ParseOptions(options)

    # Test when the source is missing.
    expected_error = 'Missing source path.'
    # pylint: disable=deprecated-method
    with self.assertRaisesRegexp(errors.BadConfigOption, expected_error):
      test_tool.ParseOptions(options)

    with shared_test_lib.TempDirectory() as temp_directory:
      options.source = self._GetTestFilePath(['testdir'])
      options.write = os.path.join(temp_directory, 'dynamic.out')

      # Test when both source and output are specified.
      test_tool.ParseOptions(options)

      with open(options.write, 'w') as file_object:
        file_object.write('bogus')

      # Test when output file already exists.
      # Escape \ otherwise assertRaisesRegexp can error with:
      # error: bogus escape: '\\1'
      expected_error = 'Output file already exists: {0:s}.'.format(
          options.write.replace('\\', '\\\\'))
      # pylint: disable=deprecated-method
      with self.assertRaisesRegexp(errors.BadConfigOption, expected_error):
        test_tool.ParseOptions(options)

  def testParseArguments(self):
    """Tests the ParseArguments function"""
    output_writer = test_lib.TestOutputWriter(encoding='utf-8')
    test_tool = psteal_tool.PstealTool(output_writer=output_writer)

    # Test ParseArguments with no output file nor source.
    result = test_tool.ParseArguments()
    self.assertFalse(result)
    output = output_writer.ReadOutput()
    expected_error = 'ERROR: Output format: dynamic requires an output file'
    self.assertIn(expected_error, output)

  @shared_test_lib.skipUnlessHasTestFile(['artifacts'])
  @shared_test_lib.skipUnlessHasTestFile(['testdir'])
  def testExtractEventsFromSourceDirectory(self):
    """Tests the ExtractEventsFromSources function on a directory."""
    output_writer = test_lib.TestOutputWriter(encoding='utf-8')
    test_tool = psteal_tool.PstealTool(output_writer=output_writer)

    options = test_lib.TestOptions()
    options.artifact_definitions_path = self._GetTestFilePath(['artifacts'])
    options.quiet = True
    options.status_view_mode = 'none'
    options.source = self._GetTestFilePath(['testdir'])

    with shared_test_lib.TempDirectory() as temp_directory:
      options.write = os.path.join(temp_directory, 'unused_output.txt')
      options.storage_file = os.path.join(temp_directory, 'storage.plaso')

      test_tool.ParseOptions(options)

      test_tool.ExtractEventsFromSources()

      expected_output = [
          b'',
          b'Source path\t: {0:s}'.format(options.source.encode('utf-8')),
          b'Source type\t: directory',
          b'Processing time\t: 00:00:00',
          b'',
          b'Processing started.',
          b'Processing completed.',
          b'',
          b'']

      output = output_writer.ReadOutput()
      self._CheckOutput(output, expected_output)

  @shared_test_lib.skipUnlessHasTestFile(['artifacts'])
  @shared_test_lib.skipUnlessHasTestFile(['bdetogo.raw'])
  def testExtractEventsFromSourceBDEImage(self):
    """Tests the ExtractEventsFromSources function on an image with BDE."""
    # TODO: added for testing.
    dfvfs_resolver.Resolver.key_chain.Empty()

    output_writer = test_lib.TestOutputWriter(encoding='utf-8')
    test_tool = psteal_tool.PstealTool(output_writer=output_writer)

    options = test_lib.TestOptions()
    options.artifact_definitions_path = self._GetTestFilePath(['artifacts'])
    options.credentials = ['password:{0:s}'.format(self._BDE_PASSWORD)]
    options.quiet = True
    options.source = self._GetTestFilePath(['bdetogo.raw'])
    options.status_view_mode = 'none'

    with shared_test_lib.TempDirectory() as temp_directory:
      options.write = os.path.join(temp_directory, 'unused_output.txt')
      options.storage_file = os.path.join(temp_directory, 'storage.plaso')

      test_tool.ParseOptions(options)

      test_tool.ExtractEventsFromSources()

      expected_output = [
          b'',
          b'Source path\t: {0:s}'.format(options.source.encode('utf-8')),
          b'Source type\t: storage media image',
          b'Processing time\t: 00:00:00',
          b'',
          b'Processing started.',
          b'Processing completed.',
          b'',
          b'']

      output = output_writer.ReadOutput()
      self._CheckOutput(output, expected_output)

  @shared_test_lib.skipUnlessHasTestFile(['artifacts'])
  @shared_test_lib.skipUnlessHasTestFile(['ímynd.dd'])
  def testExtractEventsFromSourcesImage(self):
    """Tests the ExtractEventsFromSources function on a single partition."""
    output_writer = test_lib.TestOutputWriter(encoding='utf-8')
    test_tool = psteal_tool.PstealTool(output_writer=output_writer)

    options = test_lib.TestOptions()
    options.artifact_definitions_path = self._GetTestFilePath(['artifacts'])
    options.quiet = True
    options.status_view_mode = 'none'
    options.source = self._GetTestFilePath(['ímynd.dd'])

    with shared_test_lib.TempDirectory() as temp_directory:
      options.write = os.path.join(temp_directory, 'unused_output.txt')
      options.storage_file = os.path.join(temp_directory, 'storage.plaso')

      test_tool.ParseOptions(options)

      test_tool.ExtractEventsFromSources()

      expected_output = [
          b'',
          b'Source path\t: {0:s}'.format(options.source.encode('utf-8')),
          b'Source type\t: storage media image',
          b'Processing time\t: 00:00:00',
          b'',
          b'Processing started.',
          b'Processing completed.',
          b'',
          b'']

      output = output_writer.ReadOutput()
      self._CheckOutput(output, expected_output)

  @shared_test_lib.skipUnlessHasTestFile(['artifacts'])
  @shared_test_lib.skipUnlessHasTestFile(['multi_partition_image.vmdk'])
  def testExtractEventsFromSourcePartitionedImage(self):
    """Tests the ExtractEventsFromSources function on a partitioned image."""
    output_writer = test_lib.TestOutputWriter(encoding='utf-8')
    test_tool = psteal_tool.PstealTool(output_writer=output_writer)

    options = test_lib.TestOptions()
    options.artifact_definitions_path = self._GetTestFilePath(['artifacts'])
    options.partitions = 'all'
    options.quiet = True
    options.status_view_mode = 'none'
    # Note that the source file is a RAW (VMDK flat) image.
    options.source = self._GetTestFilePath(['multi_partition_image.vmdk'])

    with shared_test_lib.TempDirectory() as temp_directory:
      options.write = os.path.join(temp_directory, 'unused_output.txt')
      options.storage_file = os.path.join(temp_directory, 'storage.plaso')

      test_tool.ParseOptions(options)

      test_tool.ExtractEventsFromSources()

      expected_output = [
          b'',
          b'Source path\t: {0:s}'.format(options.source.encode('utf-8')),
          b'Source type\t: storage media image',
          b'Processing time\t: 00:00:00',
          b'',
          b'Processing started.',
          b'Processing completed.',
          b'',
          b'']

      output = output_writer.ReadOutput()
      self._CheckOutput(output, expected_output)

  @shared_test_lib.skipUnlessHasTestFile(['artifacts'])
  @shared_test_lib.skipUnlessHasTestFile(['vsstest.qcow2'])
  def testExtractEventsFromSourceVSSImage(self):
    """Tests the ExtractEventsFromSources function on an image with VSS."""
    output_writer = test_lib.TestOutputWriter(encoding='utf-8')
    test_tool = psteal_tool.PstealTool(output_writer=output_writer)

    options = test_lib.TestOptions()
    options.artifact_definitions_path = self._GetTestFilePath(['artifacts'])
    options.quiet = True
    options.single_process = True
    options.status_view_mode = 'none'
    options.source = self._GetTestFilePath(['vsstest.qcow2'])
    options.vss_stores = 'all'

    with shared_test_lib.TempDirectory() as temp_directory:
      options.write = os.path.join(temp_directory, 'unused_output.txt')
      options.storage_file = os.path.join(temp_directory, 'storage.plaso')

      test_tool.ParseOptions(options)

      test_tool.ExtractEventsFromSources()

      expected_output = [
          b'',
          b'Source path\t: {0:s}'.format(options.source.encode('utf-8')),
          b'Source type\t: storage media image',
          b'Processing time\t: 00:00:00',
          b'',
          b'Processing started.',
          b'Processing completed.',
          b'',
          b'Number of errors encountered while extracting events: 1.',
          b'',
          b'Use pinfo to inspect errors in more detail.',
          b'',
          b'']

      output = output_writer.ReadOutput()
      self._CheckOutput(output, expected_output)

  @shared_test_lib.skipUnlessHasTestFile(['artifacts'])
  @shared_test_lib.skipUnlessHasTestFile(['System.evtx'])
  def testExtractEventsFromSourceSingleFile(self):
    """Tests the ExtractEventsFromSources function on a single file."""
    output_writer = test_lib.TestOutputWriter(encoding='utf-8')
    test_tool = psteal_tool.PstealTool(output_writer=output_writer)

    options = test_lib.TestOptions()
    options.artifact_definitions_path = self._GetTestFilePath(['artifacts'])
    options.quiet = True
    options.status_view_mode = 'none'
    options.source = self._GetTestFilePath(['System.evtx'])

    with shared_test_lib.TempDirectory() as temp_directory:
      options.write = os.path.join(temp_directory, 'unused_output.txt')
      options.storage_file = os.path.join(temp_directory, 'storage.plaso')

      test_tool.ParseOptions(options)

      test_tool.ExtractEventsFromSources()

      expected_output = [
          b'',
          b'Source path\t: {0:s}'.format(options.source.encode('utf-8')),
          b'Source type\t: single file',
          b'Processing time\t: 00:00:00',
          b'',
          b'Processing started.',
          b'Processing completed.',
          b'',
          b'']

      output = output_writer.ReadOutput()
      self._CheckOutput(output, expected_output)

  @shared_test_lib.skipUnlessHasTestFile(['artifacts'])
  @shared_test_lib.skipUnlessHasTestFile(['psort_test.plaso'])
  @shared_test_lib.skipUnlessHasTestFile(['end_to_end', 'dynamic.log'])
  def testProcessStorage(self):
    """Test the AnalyzeEvents function"""
    output_writer = test_lib.TestOutputWriter(encoding='utf-8')
    test_tool = psteal_tool.PstealTool(output_writer=output_writer)

    options = test_lib.TestOptions()
    options.artifact_definitions_path = self._GetTestFilePath(['artifacts'])
    options.storage_file = self._GetTestFilePath(['psort_test.plaso'])
    options.source = 'unused_source'

    with shared_test_lib.TempDirectory() as temp_directory:
      options.write = os.path.join(temp_directory, 'output.txt')

      test_tool.ParseOptions(options)
      test_tool.AnalyzeEvents()

      expected_output_file_name = self._GetTestFilePath(
          ['end_to_end', 'dynamic.log'])
      with open(expected_output_file_name, 'rb') as file_object:
        expected_output = file_object.read()

      with open(options.write, 'rb') as file_object:
        result_output = file_object.read()

      expected_output = sorted(expected_output.split(b'\n'))
      result_output = sorted(result_output.split(b'\n'))
      self.assertEqual(expected_output, result_output)

    output = output_writer.ReadOutput()
    self.assertIn('Events processed : 38', output)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Twitter on iOS 8+ database formatter."""

from __future__ import unicode_literals

from plaso.formatters import interface
from plaso.formatters import manager
from plaso.lib import errors


class TwitterIOSContactFormatter(interface.ConditionalEventFormatter):
  """Twitter on iOS 8+ contact event formatter."""

  DATA_TYPE = 'twitter:ios:contact'

  FORMAT_STRING_PIECES = [
      'Screen name: {screen_name}',
      'Profile picture URL: {profile_url}',
      'Name: {name}',
      'Location: {location}',
      'Description: {description}',
      'URL: {url}',
      'Following: {following}',
      'Number of followers: {followers_count}',
      'Number of following: {following_count}',
  ]

  FORMAT_STRING_SHORT_PIECES = [
      'Screen name: {screen_name}',
      'Description: {description}',
      'URL: {url}',
  ]

  SOURCE_LONG = 'Twitter iOS Contacts'
  SOURCE_SHORT = 'Twitter iOS'

  _FOLLOWING = {
      0: 'No',
      1: 'Yes',
  }

  def GetMessages(self, unused_formatter_mediator, event):
    """Determines the formatted message strings for an event object.

    Args:
      formatter_mediator (FormatterMediator): mediates the interactions between
          formatters and other components, such as storage and Windows EventLog
          resources.
      event (EventObject): event.

    Returns:
      tuple(str, str): formatted message string and short message string.

    Raises:
      WrongFormatter: if the event object cannot be formatted by the formatter.
    """
    if self.DATA_TYPE != event.data_type:
      raise errors.WrongFormatter('Unsupported data type: {0:s}.'.format(
          event.data_type))

    event_values = event.CopyToDict()

    following = event_values.get('following', None)
    if following is not None:
      event_values['following'] = (
          self._FOLLOWING.get(following, 'UNKNOWN'))

    return self._ConditionalFormatMessages(event_values)


class TwitterIOSStatusFormatter(interface.ConditionalEventFormatter):
  """Twitter on iOS 8+ status event formatter."""

  DATA_TYPE = 'twitter:ios:status'

  FORMAT_STRING_PIECES = [
      'Name: {name}',
      'User Id: {user_id}',
      'Message: {text}',
      'Favorite: {favorited}',
      'Retweet Count: {retweet_count}',
      'Favorite Count: {favorite_count}',
  ]

  FORMAT_STRING_SHORT_PIECES = [
      'Name: {name}',
      'Message: {text}',
  ]

  SOURCE_LONG = 'Twitter iOS Status'
  SOURCE_SHORT = 'Twitter iOS'

  _FAVORITED = {
      0: 'No',
      1: 'Yes',
  }

  def GetMessages(self, unused_formatter_mediator, event):
    """Determines the formatted message strings for an event object.

    Args:
      formatter_mediator (FormatterMediator): mediates the interactions between
          formatters and other components, such as storage and Windows EventLog
          resources.
      event (EventObject): event.

    Returns:
      tuple(str, str): formatted message string and short message string.

    Raises:
      WrongFormatter: if the event object cannot be formatted by the formatter.
    """
    if self.DATA_TYPE != event.data_type:
      raise errors.WrongFormatter('Unsupported data type: {0:s}.'.format(
          event.data_type))

    event_values = event.CopyToDict()

    favorited = event_values.get('favorited', None)
    if favorited is not None:
      event_values['favorited'] = (
          self._FAVORITED.get(favorited, 'UNKNOWN'))

    return self._ConditionalFormatMessages(event_values)


manager.FormattersManager.RegisterFormatters([
    TwitterIOSContactFormatter, TwitterIOSStatusFormatter])



# -*- coding: utf-8 -*-
"""Merge reader for SQLite storage files."""

from __future__ import unicode_literals

import os
import sqlite3
import zlib

from plaso.containers import errors
from plaso.containers import event_sources
from plaso.containers import events
from plaso.containers import reports
from plaso.containers import tasks
from plaso.lib import definitions
from plaso.storage import identifiers
from plaso.storage import interface


class SQLiteStorageMergeReader(interface.StorageFileMergeReader):
  """SQLite-based storage file reader for merging."""

  _CONTAINER_TYPE_ANALYSIS_REPORT = reports.AnalysisReport.CONTAINER_TYPE
  _CONTAINER_TYPE_EVENT = events.EventObject.CONTAINER_TYPE
  _CONTAINER_TYPE_EVENT_DATA = events.EventData.CONTAINER_TYPE
  _CONTAINER_TYPE_EVENT_SOURCE = event_sources.EventSource.CONTAINER_TYPE
  _CONTAINER_TYPE_EVENT_TAG = events.EventTag.CONTAINER_TYPE
  _CONTAINER_TYPE_EXTRACTION_ERROR = errors.ExtractionError.CONTAINER_TYPE
  _CONTAINER_TYPE_TASK_COMPLETION = tasks.TaskCompletion.CONTAINER_TYPE
  _CONTAINER_TYPE_TASK_START = tasks.TaskStart.CONTAINER_TYPE

  # Some container types reference other container types, such as event
  # referencing event_data. Container types in this tuple must be ordered after
  # all the container types they reference.
  _CONTAINER_TYPES = (
      _CONTAINER_TYPE_EVENT_SOURCE,
      _CONTAINER_TYPE_EVENT_DATA,
      _CONTAINER_TYPE_EVENT,
      _CONTAINER_TYPE_EVENT_TAG,
      _CONTAINER_TYPE_EXTRACTION_ERROR,
      _CONTAINER_TYPE_ANALYSIS_REPORT)

  _ADD_CONTAINER_TYPE_METHODS = {
      _CONTAINER_TYPE_ANALYSIS_REPORT: '_AddAnalysisReport',
      _CONTAINER_TYPE_EVENT: '_AddEvent',
      _CONTAINER_TYPE_EVENT_DATA: '_AddEventData',
      _CONTAINER_TYPE_EVENT_SOURCE: '_AddEventSource',
      _CONTAINER_TYPE_EVENT_TAG: '_AddEventTag',
      _CONTAINER_TYPE_EXTRACTION_ERROR: '_AddError',
  }

  _TABLE_NAMES_QUERY = (
      'SELECT name FROM sqlite_master WHERE type = "table"')

  def __init__(self, storage_writer, path):
    """Initializes a storage merge reader.

    Args:
      storage_writer (StorageWriter): storage writer.
      path (str): path to the input file.

    Raises:
      IOError: if the input file cannot be opened.
      RuntimeError: if an add container method is missing.
    """
    super(SQLiteStorageMergeReader, self).__init__(storage_writer)
    self._active_container_type = None
    self._active_cursor = None
    self._add_active_container_method = None
    self._add_container_type_methods = {}
    self._compression_format = definitions.COMPRESSION_FORMAT_NONE
    self._connection = None
    self._container_types = None
    self._cursor = None
    self._event_data_identifier_mappings = {}
    self._path = path

    # Create a runtime lookup table for the add container type method. This
    # prevents having to create a series of if-else checks for container types.
    # The table is generated at runtime as there are no forward function
    # declarations in Python.
    for container_type, method_name in self._ADD_CONTAINER_TYPE_METHODS.items():
      method = getattr(self, method_name, None)
      if not method:
        raise RuntimeError(
            'Add method missing for container type: {0:s}'.format(
                container_type))

      self._add_container_type_methods[container_type] = method

  def _AddAnalysisReport(self, analysis_report):
    """Adds an analysis report.

    Args:
      analysis_report (AnalysisReport): analysis report.
    """
    self._storage_writer.AddAnalysisReport(analysis_report)

  def _AddError(self, error):
    """Adds an error.

    Args:
      error (ExtractionError): error.
    """
    self._storage_writer.AddError(error)

  def _AddEvent(self, event):
    """Adds an event.

    Args:
      event (EventObject): event.
    """
    if hasattr(event, 'event_data_row_identifier'):
      event_data_identifier = identifiers.SQLTableIdentifier(
          self._CONTAINER_TYPE_EVENT_DATA,
          event.event_data_row_identifier)
      lookup_key = event_data_identifier.CopyToString()

      event_data_identifier = self._event_data_identifier_mappings[lookup_key]
      event.SetEventDataIdentifier(event_data_identifier)

    # TODO: add event identifier mappings for event tags.

    self._storage_writer.AddEvent(event)

  def _AddEventData(self, event_data):
    """Adds event data.

    Args:
      event_data (EventData): event data.
    """
    identifier = event_data.GetIdentifier()
    lookup_key = identifier.CopyToString()

    self._storage_writer.AddEventData(event_data)

    identifier = event_data.GetIdentifier()
    self._event_data_identifier_mappings[lookup_key] = identifier

  def _AddEventSource(self, event_source):
    """Adds an event source.

    Args:
      event_source (EventSource): event source.
    """
    self._storage_writer.AddEventSource(event_source)

  def _AddEventTag(self, event_tag):
    """Adds an event tag.

    Args:
      event_tag (EventTag): event tag.
    """
    self._storage_writer.AddEventTag(event_tag)

  def _Close(self):
    """Closes the task storage after reading."""
    self._connection.close()
    self._connection = None
    self._cursor = None

  def _GetContainerTypes(self):
    """Retrieves the container types to merge.

    Container types not defined in _CONTAINER_TYPES are ignored and not merged.

    Specific container types reference other container types, such
    as event referencing event data. The names are ordered to ensure the
    attribute containers are merged in the correct order.

    Returns:
      list[str]: names of the container types to merge.
    """
    self._cursor.execute(self._TABLE_NAMES_QUERY)
    table_names = [row[0] for row in self._cursor.fetchall()]

    return [
        table_name for table_name in self._CONTAINER_TYPES
        if table_name in table_names]

  def _Open(self):
    """Opens the task storage for reading."""
    self._connection = sqlite3.connect(
        self._path, detect_types=sqlite3.PARSE_DECLTYPES|sqlite3.PARSE_COLNAMES)
    self._cursor = self._connection.cursor()

  def _ReadStorageMetadata(self):
    """Reads the task storage metadata."""
    query = 'SELECT key, value FROM metadata'
    self._cursor.execute(query)

    metadata_values = {row[0]: row[1] for row in self._cursor.fetchall()}

    self._compression_format = metadata_values['compression_format']

  def _PrepareForNextContainerType(self):
    """Prepares for the next container type.

    This method prepares the task storage for merging the next container type.
    It set the active container type, its add method and active cursor
    accordingly.
    """
    self._active_container_type = self._container_types.pop(0)

    self._add_active_container_method = self._add_container_type_methods.get(
        self._active_container_type)

    query = 'SELECT _identifier, _data FROM {0:s}'.format(
        self._active_container_type)
    self._cursor.execute(query)

    self._active_cursor = self._cursor

  def MergeAttributeContainers(
      self, callback=None, maximum_number_of_containers=0):
    """Reads attribute containers from a task storage file into the writer.

    Args:
      callback (function[StorageWriter, AttributeContainer]): function to call
          after each attribute container is deserialized.
      maximum_number_of_containers (Optional[int]): maximum number of
          containers to merge, where 0 represent no limit.

    Returns:
      bool: True if the entire task storage file has been merged.

    Raises:
      RuntimeError: if the add method for the active attribute container
          type is missing.
      OSError: if the task storage file cannot be deleted.
    """
    if not self._cursor:
      self._Open()
      self._ReadStorageMetadata()
      self._container_types = self._GetContainerTypes()

    number_of_containers = 0
    while self._active_cursor or self._container_types:
      if not self._active_cursor:
        self._PrepareForNextContainerType()

      if maximum_number_of_containers > 0:
        number_of_rows = maximum_number_of_containers - number_of_containers
        rows = self._active_cursor.fetchmany(size=number_of_rows)
      else:
        rows = self._active_cursor.fetchall()

      if not rows:
        self._active_cursor = None
        continue

      for row in rows:
        identifier = identifiers.SQLTableIdentifier(
            self._active_container_type, row[0])

        if self._compression_format == definitions.COMPRESSION_FORMAT_ZLIB:
          serialized_data = zlib.decompress(row[1])
        else:
          serialized_data = row[1]

        attribute_container = self._DeserializeAttributeContainer(
            self._active_container_type, serialized_data)
        attribute_container.SetIdentifier(identifier)

        if self._active_container_type == self._CONTAINER_TYPE_EVENT_TAG:
          event_identifier = identifiers.SQLTableIdentifier(
              self._CONTAINER_TYPE_EVENT,
              attribute_container.event_row_identifier)
          attribute_container.SetEventIdentifier(event_identifier)

          del attribute_container.event_row_identifier

        if callback:
          callback(self._storage_writer, attribute_container)

        self._add_active_container_method(attribute_container)

        number_of_containers += 1

      if (maximum_number_of_containers > 0 and
          number_of_containers >= maximum_number_of_containers):
        return False

    self._Close()

    os.remove(self._path)

    return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests for the TLN output class."""

from __future__ import unicode_literals

import unittest

from plaso.formatters import manager as formatters_manager
from plaso.output import tln

from tests.cli import test_lib as cli_test_lib
from tests.output import test_lib


class TLNOutputModuleTest(test_lib.OutputModuleTestCase):
  """Tests for the TLN output module."""

  def setUp(self):
    """Makes preparations before running an individual test."""
    self._output_writer = cli_test_lib.TestOutputWriter()
    output_mediator = self._CreateOutputMediator()
    self._output_module = tln.TLNOutputModule(output_mediator)
    self._output_module.SetOutputWriter(self._output_writer)
    self._event_object = test_lib.TestEventObject()

  def testWriteHeader(self):
    """Tests the WriteHeader function."""
    expected_header = b'Time|Source|Host|User|Description\n'

    self._output_module.WriteHeader()

    header = self._output_writer.ReadOutput()
    self.assertEqual(header, expected_header)

  def testWriteEventBody(self):
    """Tests the WriteEventBody function."""
    formatters_manager.FormattersManager.RegisterFormatter(
        test_lib.TestEventFormatter)

    self._output_module.WriteEventBody(self._event_object)

    expected_event_body = (
        b'1340821021|LOG|ubuntu|root|2012-06-27T18:17:01+00:00; UNKNOWN; '
        b'Reporter <CRON> PID:  8442  (pam_unix(cron:session): '
        b'session closed for user root)\n')

    event_body = self._output_writer.ReadOutput()
    self.assertEqual(event_body, expected_event_body)

    self.assertEqual(event_body.count(b'|'), 4)

    formatters_manager.FormattersManager.DeregisterFormatter(
        test_lib.TestEventFormatter)


class L2TTLNOutputModuleTest(test_lib.OutputModuleTestCase):
  """Tests for the log2timeline TLN output module."""

  def setUp(self):
    """Makes preparations before running an individual test."""
    output_mediator = self._CreateOutputMediator()
    self._output_writer = cli_test_lib.TestOutputWriter()
    self._output_module = tln.L2TTLNOutputModule(output_mediator)
    self._output_module.SetOutputWriter(self._output_writer)
    self._event_object = test_lib.TestEventObject()

  def testWriteHeader(self):
    """Tests the WriteHeader function."""
    expected_header = b'Time|Source|Host|User|Description|TZ|Notes\n'

    self._output_module.WriteHeader()

    header = self._output_writer.ReadOutput()
    self.assertEqual(header, expected_header)

  def testWriteEventBody(self):
    """Tests the WriteEventBody function."""
    formatters_manager.FormattersManager.RegisterFormatter(
        test_lib.TestEventFormatter)

    self._output_module.WriteEventBody(self._event_object)

    expected_event_body = (
        b'1340821021|LOG|ubuntu|root|2012-06-27T18:17:01+00:00; UNKNOWN; '
        b'Reporter <CRON> PID:  8442  (pam_unix(cron:session): '
        b'session closed for user root)'
        b'|UTC|File: OS: /var/log/syslog.1 inode: 12345678\n')

    event_body = self._output_writer.ReadOutput()
    self.assertEqual(event_body, expected_event_body)

    self.assertEqual(event_body.count(b'|'), 6)

    formatters_manager.FormattersManager.DeregisterFormatter(
        test_lib.TestEventFormatter)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests for the 4n6time output modules shared CLI arguments helper."""

from __future__ import unicode_literals

import argparse
import unittest

from plaso.cli.helpers import shared_4n6time_output
from plaso.lib import errors
from plaso.output import shared_4n6time

from tests.cli import test_lib as cli_test_lib
from tests.cli.helpers import test_lib


class Shared4n6TimeOutputArgumentsHelperTest(
    test_lib.OutputModuleArgumentsHelperTest):
  """Tests the 4n6time output modules shared CLI arguments helper."""

  # pylint: disable=no-member,protected-access

  _EXPECTED_OUTPUT = """\
usage: cli_helper.py [--append] [--evidence EVIDENCE] [--fields FIELDS]
                     [--additional_fields ADDITIONAL_FIELDS]

Test argument parser.

optional arguments:
  --additional_fields ADDITIONAL_FIELDS
                        Defines extra fields to be included in the output, in
                        addition to the default fields, which are
                        datetime,host,source,sourcetype,user,type.
  --append              Defines whether the intention is to append to an
                        already existing database or overwrite it. Defaults to
                        overwrite.
  --evidence EVIDENCE   Set the evidence field to a specific value, defaults
                        to empty.
  --fields FIELDS       Defines which fields should be indexed in the
                        database.
"""

  def testAddArguments(self):
    """Tests the AddArguments function."""
    argument_parser = argparse.ArgumentParser(
        prog='cli_helper.py',
        description='Test argument parser.', add_help=False,
        formatter_class=cli_test_lib.SortedArgumentsHelpFormatter)

    shared_4n6time_output.Shared4n6TimeOutputArgumentsHelper.AddArguments(
        argument_parser)

    output = self._RunArgparseFormatHelp(argument_parser)
    self.assertEqual(output, self._EXPECTED_OUTPUT)

  def testParseOptions(self):
    """Tests the ParseOptions function."""
    options = cli_test_lib.TestOptions()

    output_mediator = self._CreateOutputMediator()
    output_module = shared_4n6time.Shared4n6TimeOutputModule(output_mediator)
    shared_4n6time_output.Shared4n6TimeOutputArgumentsHelper.ParseOptions(
        options, output_module)

    with self.assertRaises(errors.BadConfigObject):
      shared_4n6time_output.Shared4n6TimeOutputArgumentsHelper.ParseOptions(
          options, None)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests for the Windows Registry file parser."""

from __future__ import unicode_literals

import unittest

from plaso.parsers import winreg
# Register all plugins.
from plaso.parsers import winreg_plugins  # pylint: disable=unused-import

from tests import test_lib as shared_test_lib
from tests.parsers import test_lib


class WinRegistryParserTest(test_lib.ParserTestCase):
  """Tests for the Windows Registry file parser."""

  # pylint: disable=protected-access

  def _GetParserChains(self, events):
    """Return a dict with a plugin count given a list of events."""
    parser_chains = {}
    for event in events:
      parser_chain = getattr(event, 'parser', None)
      if not parser_chain:
        continue

      if parser_chain in parser_chains:
        parser_chains[parser_chain] += 1
      else:
        parser_chains[parser_chain] = 1

    return parser_chains

  def _PluginNameToParserChain(self, plugin_name):
    """Generate the correct parser chain for a given plugin."""
    return 'winreg/{0:s}'.format(plugin_name)

  def testEnablePlugins(self):
    """Tests the EnablePlugins function."""
    parser = winreg.WinRegistryParser()
    parser.EnablePlugins(['appcompatcache'])

    self.assertIsNotNone(parser)
    self.assertIsNotNone(parser._default_plugin)
    self.assertNotEqual(parser._plugins, [])
    self.assertEqual(len(parser._plugins), 1)

  @shared_test_lib.skipUnlessHasTestFile(['NTUSER.DAT'])
  def testParseNTUserDat(self):
    """Tests the Parse function on a NTUSER.DAT file."""
    parser = winreg.WinRegistryParser()
    storage_writer = self._ParseFile(['NTUSER.DAT'], parser)

    events = list(storage_writer.GetEvents())

    parser_chains = self._GetParserChains(events)

    expected_parser_chain = self._PluginNameToParserChain('userassist')
    self.assertTrue(expected_parser_chain in parser_chains)

    self.assertEqual(parser_chains[expected_parser_chain], 14)

  @shared_test_lib.skipUnlessHasTestFile(['ntuser.dat.LOG'])
  def testParseNoRootKey(self):
    """Test the parse function on a Registry file with no root key."""
    parser = winreg.WinRegistryParser()
    storage_writer = self._ParseFile(['ntuser.dat.LOG'], parser)

    self.assertEqual(storage_writer.number_of_events, 0)

  @shared_test_lib.skipUnlessHasTestFile(['SYSTEM'])
  def testParseSystem(self):
    """Tests the Parse function on a SYSTEM file."""
    parser = winreg.WinRegistryParser()
    storage_writer = self._ParseFile(['SYSTEM'], parser)

    events = list(storage_writer.GetEvents())

    parser_chains = self._GetParserChains(events)

    # Check the existence of few known plugins, see if they
    # are being properly picked up and are parsed.
    plugin_names = [
        'windows_usbstor_devices', 'windows_boot_execute',
        'windows_services']
    for plugin in plugin_names:
      expected_parser_chain = self._PluginNameToParserChain(plugin)
      self.assertTrue(
          expected_parser_chain in parser_chains,
          'Chain {0:s} not found in events.'.format(expected_parser_chain))

    # Check that the number of events produced by each plugin are correct.
    parser_chain = self._PluginNameToParserChain('windows_usbstor_devices')
    self.assertEqual(parser_chains.get(parser_chain, 0), 10)

    parser_chain = self._PluginNameToParserChain('windows_boot_execute')
    self.assertEqual(parser_chains.get(parser_chain, 0), 4)

    parser_chain = self._PluginNameToParserChain('windows_services')
    self.assertEqual(parser_chains.get(parser_chain, 0), 831)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The SCCM log formatter."""

from __future__ import unicode_literals

from plaso.formatters import interface
from plaso.formatters import manager


class SCCMEventFormatter(interface.ConditionalEventFormatter):
  """Class for SCCM event formatter."""

  DATA_TYPE = 'software_management:sccm:log'

  FORMAT_STRING_SEPARATOR = ' '

  FORMAT_STRING_PIECES = [
      '{component}',
      '{text}']

  FORMAT_STRING_SHORT_PIECES = ['{text}']

  SOURCE_LONG = 'SCCM Event'
  SOURCE_SHORT = 'LOG'


manager.FormattersManager.RegisterFormatter(SCCMEventFormatter)



# -*- coding: utf-8 -*-
"""Output module for the Excel Spreadsheet (XLSX) output format."""

from __future__ import unicode_literals

import datetime
import os
import re

try:
  import xlsxwriter
except ImportError:
  xlsxwriter = None

from plaso.lib import py2to3
from plaso.lib import timelib
from plaso.output import dynamic
from plaso.output import interface
from plaso.output import manager


class XLSXOutputModule(interface.OutputModule):
  """Output module for the Excel Spreadsheet (XLSX) output format."""

  NAME = 'xlsx'
  DESCRIPTION = 'Excel Spreadsheet (XLSX) output'

  _DEFAULT_FIELDS = [
      'datetime', 'timestamp_desc', 'source', 'source_long',
      'message', 'parser', 'display_name', 'tag']

  _DEFAULT_TIMESTAMP_FORMAT = 'YYYY-MM-DD HH:MM:SS.000'

  _MAX_COLUMN_WIDTH = 50
  _MIN_COLUMN_WIDTH = 6

  # Illegal Unicode characters for XML.
  _ILLEGAL_XML_RE = re.compile((
      r'[\x00-\x08\x0b-\x1f\x7f-\x84\x86-\x9f\ud800-\udfff\ufdd0-\ufddf'
      r'\ufffe-\uffff]'))

  def __init__(self, output_mediator):
    """Initializes an Excel Spreadsheet (XLSX) output module.

    Args:
      output_mediator (OutputMediator): output mediator.
    """
    super(XLSXOutputModule, self).__init__(output_mediator)
    self._column_widths = {}
    self._current_row = 0
    self._dynamic_fields_helper = dynamic.DynamicFieldsHelper(output_mediator)
    self._fields = self._DEFAULT_FIELDS
    self._filename = None
    self._sheet = None
    self._timestamp_format = self._DEFAULT_TIMESTAMP_FORMAT
    self._workbook = None

  def _FormatDateTime(self, event):
    """Formats the date to a datetime object without timezone information.

    Note: timezone information must be removed due to lack of support
    by xlsxwriter and Excel.

    Args:
      event (EventObject): event.

    Returns:
      datetime.datetime:str: date and time value or a string containing
          "ERROR" on OverflowError.
    """
    try:
      timestamp = timelib.Timestamp.CopyToDatetime(
          event.timestamp, self._output_mediator.timezone, raise_error=True)

      return timestamp.replace(tzinfo=None)

    except (OverflowError, ValueError) as exception:
      self._ReportEventError(event, (
          'unable to copy timestamp: {0!s} to a human readable date and time '
          'with error: {1!s}. Defaulting to: "ERROR"').format(
              event.timestamp, exception))
      return 'ERROR'

  def _RemoveIllegalXMLCharacters(self, xml_string):
    """Removes illegal characters for XML.

    If the input is not a string it will be returned unchanged.

    Args:
      xml_string (str): XML with possible illegal characters.

    Returns:
      str: XML where all illegal characters have been removed.
    """
    if not isinstance(xml_string, py2to3.STRING_TYPES):
      return xml_string

    return self._ILLEGAL_XML_RE.sub('\ufffd', xml_string)

  def Close(self):
    """Closes the output."""
    self._workbook.close()

  def Open(self):
    """Creates a new workbook.

    Raises:
      IOError: if the specified output file already exists.
      ValueError: if the filename is not set.
    """
    if not self._filename:
      raise ValueError('Missing filename.')

    if os.path.isfile(self._filename):
      raise IOError((
          'Unable to use an already existing file for output '
          '[{0:s}]').format(self._filename))

    options = {
        'constant_memory': True,
        'strings_to_urls': False,
        'strings_to_formulas': False,
        'default_date_format': self._timestamp_format}
    self._workbook = xlsxwriter.Workbook(self._filename, options)
    self._sheet = self._workbook.add_worksheet('Sheet')
    self._current_row = 0

  def SetFields(self, fields):
    """Sets the fields to output.

    Args:
      fields (list[str]): names of the fields to output.
    """
    self._fields = fields

  def SetFilename(self, filename):
    """Sets the filename.

    Args:
      filename (str): filename.
    """
    self._filename = filename

  def SetTimestampFormat(self, timestamp_format):
    """Set the timestamp format to use for the datetime column.

    Args:
      timestamp_format (str): format string of date and time values.
    """
    self._timestamp_format = timestamp_format

  def WriteEventBody(self, event):
    """Writes the body of an event object to the spreadsheet.

    Args:
      event (EventObject): event.
    """
    for field_name in self._fields:
      if field_name == 'datetime':
        output_value = self._FormatDateTime(event)
      else:
        output_value = self._dynamic_fields_helper.GetFormattedField(
            event, field_name)

      output_value = self._RemoveIllegalXMLCharacters(output_value)

      # Auto adjust the column width based on the length of the output value.
      column_index = self._fields.index(field_name)
      self._column_widths.setdefault(column_index, 0)

      if field_name == 'datetime':
        column_width = min(
            self._MAX_COLUMN_WIDTH, len(self._timestamp_format) + 2)
      else:
        column_width = min(self._MAX_COLUMN_WIDTH, len(output_value) + 2)

      self._column_widths[column_index] = max(
          self._MIN_COLUMN_WIDTH, self._column_widths[column_index],
          column_width)
      self._sheet.set_column(
          column_index, column_index, self._column_widths[column_index])

      if (field_name == 'datetime'
          and isinstance(output_value, datetime.datetime)):
        self._sheet.write_datetime(
            self._current_row, column_index, output_value)
      else:
        self._sheet.write(self._current_row, column_index, output_value)

    self._current_row += 1

  def WriteHeader(self):
    """Writes the header to the spreadsheet."""
    self._column_widths = {}
    bold = self._workbook.add_format({'bold': True})
    bold.set_align('center')
    for index, field_name in enumerate(self._fields):
      self._sheet.write(self._current_row, index, field_name, bold)
      self._column_widths[index] = len(field_name) + 2
    self._current_row += 1
    self._sheet.autofilter(0, len(self._fields) - 1, 0, 0)
    self._sheet.freeze_panes(1, 0)


manager.OutputManager.RegisterOutput(
    XLSXOutputModule, disabled=xlsxwriter is None)



# -*- coding: utf-8 -*-
"""CLI related functions and classes for testing."""

from __future__ import unicode_literals

import argparse
import io
import operator
import os

from plaso.cli import tools

from tests import test_lib as shared_test_lib


class SortedArgumentsHelpFormatter(argparse.HelpFormatter):
  """Class that implements an argparse help formatter with sorted arguments."""

  def add_arguments(self, actions):
    """Adds arguments.

    Args:
      actions (TODO): TODO.
    """
    actions = sorted(actions, key=operator.attrgetter('option_strings'))
    super(SortedArgumentsHelpFormatter, self).add_arguments(actions)


class TestOptions(object):
  """Class to define test options."""


class TestOutputWriter(tools.FileObjectOutputWriter):
  """Class that implements a test output writer."""

  def __init__(self, encoding='utf-8'):
    """Initializes the output writer object.

    Args:
      encoding (Optional[str]): output encoding.
    """
    file_object = io.BytesIO()
    super(TestOutputWriter, self).__init__(file_object, encoding=encoding)
    self._read_offset = 0

  def ReadOutput(self):
    """Reads the newly added output data.

    Returns:
      bytes: encoded output data.
    """
    self._file_object.seek(self._read_offset, os.SEEK_SET)
    output_data = self._file_object.read()
    self._read_offset = self._file_object.tell()

    return output_data


class CLIToolTestCase(shared_test_lib.BaseTestCase):
  """The unit test case for a CLI tool."""

  def _RunArgparseFormatHelp(self, argument_parser):
    """Runs argparse.format_help() with test conditions.

    Args:
      argument_parser (argparse.ArgumentParser): argument parser.

    Returns:
      bytes: output of argparse.format_help().
    """
    columns_environment_variable = os.environ.get('COLUMNS', None)
    os.environ['COLUMNS'] = '80'

    try:
      output = argument_parser.format_help()
    finally:
      if columns_environment_variable:
        os.environ['COLUMNS'] = columns_environment_variable
      else:
        del os.environ['COLUMNS']

    return output



# -*- coding: utf-8 -*-
"""Shared code for Elasticsearch based output modules."""

from __future__ import unicode_literals

import logging

from dfvfs.serializer.json_serializer import JsonPathSpecSerializer

try:
  import elasticsearch
  from elasticsearch.exceptions import ConnectionError as ElasticConnectionError
except ImportError:
  elasticsearch = None

from plaso.lib import errors
from plaso.lib import timelib
from plaso.output import interface
from plaso.output import logger


# Configure the Elasticsearch logger.
if elasticsearch:
  elastic_logger = logging.getLogger('elasticsearch.trace')
  elastic_logger.setLevel(logging.WARNING)


class SharedElasticsearchOutputModule(interface.OutputModule):
  """Shared functionality for an Elasticsearch output module."""

  # pylint: disable=abstract-method

  NAME = 'elastic_shared'

  _DEFAULT_DOCUMENT_TYPE = 'plaso_event'

  _DEFAULT_FLUSH_INTERVAL = 1000

  # Number of seconds to wait before a request to Elasticsearch is timed out.
  _DEFAULT_REQUEST_TIMEOUT = 300

  def __init__(self, output_mediator):
    """Initializes an Elasticsearch output module.

    Args:
      output_mediator (OutputMediator): mediates interactions between output
          modules and other components, such as storage and dfvfs.
    """
    super(SharedElasticsearchOutputModule, self).__init__(output_mediator)
    self._client = None
    self._document_type = self._DEFAULT_DOCUMENT_TYPE
    self._event_documents = []
    self._flush_interval = self._DEFAULT_FLUSH_INTERVAL
    self._host = None
    self._index_name = None
    self._number_of_buffered_events = 0
    self._password = None
    self._port = None
    self._username = None

  def _Connect(self):
    """Connects to an Elasticsearch server."""
    elastic_hosts = [{'host': self._host, 'port': self._port}]

    elastic_http_auth = None
    if self._username is not None:
      elastic_http_auth = (self._username, self._password)

    self._client = elasticsearch.Elasticsearch(
        elastic_hosts, http_auth=elastic_http_auth)

    logger.debug('Connected to Elasticsearch server: {0:s} port: {1:d}.'.format(
        self._host, self._port))

  def _CreateIndexIfNotExists(self, index_name, mappings):
    """Creates an Elasticsearch index if it not already exists.

    Args:
      index_name (str): mame of the index.
      mappings (dict[str, object]): mappings of the index.

    Raises:
      RuntimeError: if the Elasticsearch index cannot be created.
    """
    try:
      if not self._client.indices.exists(index_name):
        self._client.indices.create(
            body={'mappings': mappings}, index=index_name)

    except ElasticConnectionError as exception:
      raise RuntimeError(
          'Unable to create Elasticsearch index with error: {0!s}'.format(
              exception))

  def _FlushEvents(self):
    """Inserts the buffered event documents into Elasticsearch."""
    try:
      # pylint: disable=unexpected-keyword-arg
      # pylint does not recognizes request_timeout as a valid kwarg. According
      # to http://elasticsearch-py.readthedocs.io/en/master/api.html#timeout
      # it should be supported.
      self._client.bulk(
          body=self._event_documents, doc_type=self._document_type,
          index=self._index_name, request_timeout=self._DEFAULT_REQUEST_TIMEOUT)

    except ValueError as exception:
      # Ignore problematic events
      logger.warning('Unable to bulk insert with error: {0!s}'.format(
          exception))

    logger.debug('Inserted {0:d} events into Elasticsearch'.format(
        self._number_of_buffered_events))

    self._event_documents = []
    self._number_of_buffered_events = 0

  def _GetSanitizedEventValues(self, event):
    """Sanitizes the event for use in Elasticsearch.

    The event values need to be sanitized to prevent certain values from
    causing problems when indexing with Elasticsearch. For example the path
    specification is a nested dictionary which will cause problems for
    Elasticsearch automatic indexing.

    Args:
      event (EventObject): event.

    Returns:
      dict[str, object]: sanitized event values.

    Raises:
      NoFormatterFound: if no event formatter can be found to match the data
          type in the event.
    """
    event_values = {}
    for attribute_name, attribute_value in event.GetAttributes():
      # Ignore the regvalue attribute as it cause issues when indexing.
      if attribute_name == 'regvalue':
        continue

      if attribute_name == 'pathspec':
        try:
          attribute_value = JsonPathSpecSerializer.WriteSerialized(
              attribute_value)
        except TypeError:
          continue
      event_values[attribute_name] = attribute_value

    # Add a string representation of the timestamp.
    try:
      attribute_value = timelib.Timestamp.RoundToSeconds(event.timestamp)
    except TypeError as exception:
      logger.warning((
          'Unable to round timestamp {0!s}. error: {1!s}. '
          'Defaulting to 0').format(event.timestamp, exception))
      attribute_value = 0

    attribute_value = timelib.Timestamp.CopyToIsoFormat(
        attribute_value, timezone=self._output_mediator.timezone)
    event_values['datetime'] = attribute_value

    message, _ = self._output_mediator.GetFormattedMessages(event)
    if message is None:
      data_type = getattr(event, 'data_type', 'UNKNOWN')
      raise errors.NoFormatterFound(
          'Unable to find event formatter for: {0:s}.'.format(data_type))

    event_values['message'] = message

    # Tags needs to be a list for Elasticsearch to index correctly.
    try:
      labels = list(event_values['tag'].labels)
    except (KeyError, AttributeError):
      labels = []
    event_values['tag'] = labels

    source_short, source = self._output_mediator.GetFormattedSources(
        event)
    if source is None or source_short is None:
      data_type = getattr(event, 'data_type', 'UNKNOWN')
      raise errors.NoFormatterFound(
          'Unable to find event formatter for: {0:s}.'.format(data_type))

    event_values['source_short'] = source_short
    event_values['source_long'] = source

    return event_values

  def _InsertEvent(self, event, force_flush=False):
    """Inserts an event.

    Events are buffered in the form of documents and inserted to Elasticsearch
    when either forced to flush or when the flush interval (threshold) has been
    reached.

    Args:
      event (EventObject): event.
      force_flush (bool): True if buffered event documents should be inserted
          into Elasticsearch.
    """
    if event:
      event_document = {'index': {
          '_index': self._index_name, '_type': self._document_type}}
      event_values = self._GetSanitizedEventValues(event)

      self._event_documents.append(event_document)
      self._event_documents.append(event_values)
      self._number_of_buffered_events += 1

    if force_flush or self._number_of_buffered_events > self._flush_interval:
      self._FlushEvents()

  def Close(self):
    """Closes connection to Elasticsearch.

    Inserts any remaining buffered event documents.
    """
    self._InsertEvent(None, force_flush=True)

    self._client = None

  def SetDocumentType(self, document_type):
    """Sets the document type.

    Args:
      document_type (str): document type.
    """
    self._document_type = document_type
    logger.debug('Elasticsearch document type: {0:s}'.format(document_type))

  def SetFlushInterval(self, flush_interval):
    """Set the flush interval.

    Args:
      flush_interval (int): number of events to buffer before doing a bulk
          insert.
    """
    self._flush_interval = flush_interval
    logger.debug('Elasticsearch flush interval: {0:d}'.format(flush_interval))

  def SetIndexName(self, index_name):
    """Set the index name.

    Args:
      index_name (str): name of the index.
    """
    self._index_name = index_name
    logger.debug('Elasticsearch index name: {0:s}'.format(index_name))

  def SetPassword(self, password):
    """Set the password.

    Args:
      password (str): password to authenticate with.
    """
    self._password = password
    logger.debug('Elastic password: ********')

  def SetServerInformation(self, server, port):
    """Set the server information.

    Args:
      server (str): IP address or hostname of the server.
      port (int): Port number of the server.
    """
    self._host = server
    self._port = port
    logger.debug('Elasticsearch server: {0:s} port: {1:d}'.format(server, port))

  def SetUsername(self, username):
    """Sets the username.

    Args:
      username (str): username to authenticate with.
    """
    self._username = username
    logger.debug('Elasticsearch username: {0:s}'.format(username))

  def WriteEventBody(self, event):
    """Writes an event to the output.

    Args:
      event (EventObject): event.
    """
    self._InsertEvent(event)



# -*- coding: utf-8 -*-
"""The artifact knowledge base object.

The knowledge base is filled by user provided input and the pre-processing
phase. It is intended to provide successive phases, like the parsing and
analysis phases, with essential information like e.g. the timezone and
codepage of the source data.
"""

from __future__ import unicode_literals

import codecs
import datetime

from plaso.containers import artifacts
from plaso.engine import logger
from plaso.lib import py2to3

import pytz  # pylint: disable=wrong-import-order


class KnowledgeBase(object):
  """Class that implements the artifact knowledge base."""

  CURRENT_SESSION = 0

  def __init__(self):
    """Initializes a knowledge base."""
    super(KnowledgeBase, self).__init__()
    self._codepage = 'cp1252'
    self._environment_variables = {}
    self._hostnames = {}
    self._time_zone = pytz.UTC
    self._user_accounts = {}
    self._values = {}

  @property
  def codepage(self):
    """str: codepage of the current session."""
    return self.GetValue('codepage', default_value=self._codepage)

  @property
  def hostname(self):
    """str: hostname of the current session."""
    hostname_artifact = self._hostnames.get(self.CURRENT_SESSION, None)
    if not hostname_artifact:
      return ''

    return hostname_artifact.name or ''

  @property
  def timezone(self):
    """datetime.tzinfo: timezone of the current session."""
    return self._time_zone

  @property
  def user_accounts(self):
    """list[UserAccountArtifact]: user accounts of the current session."""
    return self._user_accounts.get(self.CURRENT_SESSION, {}).values()

  @property
  def year(self):
    """int: year of the current session."""
    return self.GetValue('year', default_value=0)

  def AddUserAccount(self, user_account, session_identifier=CURRENT_SESSION):
    """Adds an user account.

    Args:
      user_account (UserAccountArtifact): user account artifact.
      session_identifier (Optional[str])): session identifier, where
          CURRENT_SESSION represents the active session.

    Raises:
      KeyError: if the user account already exists.
    """
    if session_identifier not in self._user_accounts:
      self._user_accounts[session_identifier] = {}

    user_accounts = self._user_accounts[session_identifier]
    if user_account.identifier in user_accounts:
      raise KeyError('User account: {0:s} already exists.'.format(
          user_account.identifier))

    user_accounts[user_account.identifier] = user_account

  def AddEnvironmentVariable(self, environment_variable):
    """Adds an environment variable.

    Args:
      environment_variable (EnvironmentVariableArtifact): environment variable
          artifact.

    Raises:
      KeyError: if the environment variable already exists.
    """
    name = environment_variable.name.upper()
    if name in self._environment_variables:
      raise KeyError('Environment variable: {0:s} already exists.'.format(
          environment_variable.name))

    self._environment_variables[name] = environment_variable

  def GetEnvironmentVariable(self, name):
    """Retrieves an environment variable.

    Args:
      name (str): name of the environment variable.

    Returns:
      EnvironmentVariableArtifact: environment variable artifact or None
          if there was no value set for the given name.
    """
    name = name.upper()
    return self._environment_variables.get(name, None)

  def GetEnvironmentVariables(self):
    """Retrieves the environment variables.

    Returns:
      list[EnvironmentVariableArtifact]: environment variable artifacts.
    """
    return self._environment_variables.values()

  def GetHostname(self, session_identifier=CURRENT_SESSION):
    """Retrieves the hostname related to the event.

    If the hostname is not stored in the event it is determined based
    on the preprocessing information that is stored inside the storage file.

    Args:
      session_identifier (Optional[str])): session identifier, where
          CURRENT_SESSION represents the active session.

    Returns:
      str: hostname.
    """
    hostname_artifact = self._hostnames.get(session_identifier, None)
    if not hostname_artifact:
      return ''

    return hostname_artifact.name or ''

  # TODO: remove this function it is incorrect.
  def GetStoredHostname(self):
    """Retrieves the stored hostname.

    The hostname is determined based on the preprocessing information
    that is stored inside the storage file.

    Returns:
      str: hostname.
    """
    store_number = len(self._hostnames)
    return self._hostnames.get(store_number, None)

  def GetSystemConfigurationArtifact(self, session_identifier=CURRENT_SESSION):
    """Retrieves the knowledge base as a system configuration artifact.

    Args:
      session_identifier (Optional[str])): session identifier, where
          CURRENT_SESSION represents the active session.

    Returns:
      SystemConfigurationArtifact: system configuration artifact.
    """
    system_configuration = artifacts.SystemConfigurationArtifact()

    system_configuration.code_page = self.GetValue(
        'codepage', default_value=self._codepage)

    system_configuration.hostname = self._hostnames.get(
        session_identifier, None)

    system_configuration.keyboard_layout = self.GetValue('keyboard_layout')
    system_configuration.operating_system = self.GetValue('operating_system')
    system_configuration.operating_system_product = self.GetValue(
        'operating_system_product')
    system_configuration.operating_system_version = self.GetValue(
        'operating_system_version')

    date_time = datetime.datetime(2017, 1, 1)
    time_zone = self._time_zone.tzname(date_time)

    if time_zone and isinstance(time_zone, py2to3.BYTES_TYPE):
      time_zone = time_zone.decode('ascii')

    system_configuration.time_zone = time_zone

    user_accounts = self._user_accounts.get(session_identifier, {})
    # In Python 3 dict.values() returns a type dict_values, which will cause
    # the JSON serializer to raise a TypeError.
    system_configuration.user_accounts = list(user_accounts.values())

    return system_configuration

  def GetUsernameByIdentifier(
      self, user_identifier, session_identifier=CURRENT_SESSION):
    """Retrieves the username based on an user identifier.

    Args:
      user_identifier (str): user identifier, either a UID or SID.
      session_identifier (Optional[str])): session identifier, where
          CURRENT_SESSION represents the active session.

    Returns:
      str: username.
    """
    user_accounts = self._user_accounts.get(session_identifier, {})
    user_account = user_accounts.get(user_identifier, None)
    if not user_account:
      return ''

    return user_account.username or ''

  def GetUsernameForPath(self, path):
    """Retrieves a username for a specific path.

    This is determining if a specific path is within a user's directory and
    returning the username of the user if so.

    Args:
      path (str): path.

    Returns:
      str: username or None if the path does not appear to be within a user's
          directory.
    """
    path = path.lower()

    user_accounts = self._user_accounts.get(self.CURRENT_SESSION, {})
    for user_account in iter(user_accounts.values()):
      if not user_account.user_directory:
        continue

      user_directory = user_account.user_directory.lower()
      if path.startswith(user_directory):
        return user_account.username

    return None

  def GetValue(self, identifier, default_value=None):
    """Retrieves a value by identifier.

    Args:
      identifier (str): case insensitive unique identifier for the value.
      default_value (object): default value.

    Returns:
      object: value or default value if not available.

    Raises:
      TypeError: if the identifier is not a string type.
    """
    if not isinstance(identifier, py2to3.STRING_TYPES):
      raise TypeError('Identifier not a string type.')

    identifier = identifier.lower()
    return self._values.get(identifier, default_value)

  def HasUserAccounts(self):
    """Determines if the knowledge base contains user accounts.

    Returns:
      bool: True if the knowledge base contains user accounts.
    """
    return self._user_accounts.get(self.CURRENT_SESSION, {}) != {}

  def ReadSystemConfigurationArtifact(
      self, system_configuration, session_identifier=CURRENT_SESSION):
    """Reads the knowledge base values from a system configuration artifact.

    Note that this overwrites existing values in the knowledge base.

    Args:
      system_configuration (SystemConfigurationArtifact): system configuration
          artifact.
      session_identifier (Optional[str])): session identifier, where
          CURRENT_SESSION represents the active session.
    """
    if system_configuration.code_page:
      try:
        self.SetCodepage(system_configuration.code_page)
      except ValueError:
        logger.warning(
            'Unsupported codepage: {0:s}, defaulting to {1:s}'.format(
                system_configuration.code_page, self._codepage))

    self._hostnames[session_identifier] = system_configuration.hostname

    self.SetValue('keyboard_layout', system_configuration.keyboard_layout)

    self.SetValue('operating_system', system_configuration.operating_system)
    self.SetValue(
        'operating_system_product',
        system_configuration.operating_system_product)
    self.SetValue(
        'operating_system_version',
        system_configuration.operating_system_version)

    if system_configuration.time_zone:
      try:
        self.SetTimeZone(system_configuration.time_zone)
      except ValueError:
        logger.warning(
            'Unsupported time zone: {0:s}, defaulting to {1:s}'.format(
                system_configuration.time_zone, self.timezone.zone))

    self._user_accounts[session_identifier] = {
        user_account.username: user_account
        for user_account in system_configuration.user_accounts}

  def SetCodepage(self, codepage):
    """Sets the codepage.

    Args:
      codepage (str): codepage.

    Raises:
      ValueError: if the codepage is not supported.
    """
    try:
      codecs.getencoder(codepage)
      self._codepage = codepage
    except LookupError:
      raise ValueError('Unsupported codepage: {0:s}'.format(codepage))

  def SetEnvironmentVariable(self, environment_variable):
    """Sets an environment variable.

    Args:
      environment_variable (EnvironmentVariableArtifact): environment variable
          artifact.
    """
    name = environment_variable.name.upper()
    self._environment_variables[name] = environment_variable

  def SetHostname(self, hostname, session_identifier=CURRENT_SESSION):
    """Sets a hostname.

    Args:
      hostname (HostnameArtifact): hostname artifact.
      session_identifier (Optional[str])): session identifier, where
          CURRENT_SESSION represents the active session.
    """
    self._hostnames[session_identifier] = hostname

  def SetTimeZone(self, time_zone):
    """Sets the time zone.

    Args:
      time_zone (str): time zone.

    Raises:
      ValueError: if the timezone is not supported.
    """
    try:
      self._time_zone = pytz.timezone(time_zone)
    except (AttributeError, pytz.UnknownTimeZoneError):
      raise ValueError('Unsupported timezone: {0!s}'.format(time_zone))

  def SetValue(self, identifier, value):
    """Sets a value by identifier.

    Args:
      identifier (str): case insensitive unique identifier for the value.
      value (object): value.

    Raises:
      TypeError: if the identifier is not a string type.
    """
    if not isinstance(identifier, py2to3.STRING_TYPES):
      raise TypeError('Identifier not a string type.')

    identifier = identifier.lower()
    self._values[identifier] = value



# -*- coding: utf-8 -*-
"""The Timesketch output module CLI arguments helper."""

from __future__ import unicode_literals

import uuid

from plaso.lib import errors
from plaso.cli.helpers import interface
from plaso.cli.helpers import manager
from plaso.output import timesketch_out


class TimesketchOutputArgumentsHelper(interface.ArgumentsHelper):
  """Timesketch output module CLI arguments helper."""

  NAME = 'timesketch'
  CATEGORY = 'output'
  DESCRIPTION = 'Argument helper for the timesketch output module.'

  _DEFAULT_DOCUMENT_TYPE = 'plaso_event'
  _DEFAULT_FLUSH_INTERVAL = 1000
  _DEFAULT_NAME = ''
  _DEFAULT_USERNAME = None
  _DEFAULT_UUID = '{0:s}'.format(uuid.uuid4().hex)

  @classmethod
  def AddArguments(cls, argument_group):
    """Adds command line arguments the helper supports to an argument group.

    This function takes an argument parser or an argument group object and adds
    to it all the command line arguments this helper supports.

    Args:
      argument_group (argparse._ArgumentGroup|argparse.ArgumentParser):
          argparse group.
    """
    argument_group.add_argument(
        '--name', '--timeline_name', '--timeline-name',
        dest='timeline_name', type=str, action='store',
        default=cls._DEFAULT_NAME, required=False, help=(
            'The name of the timeline in Timesketch. Default: '
            'hostname if present in the storage file. If no hostname '
            'is found then manual input is used.'))

    argument_group.add_argument(
        '--index', dest='index', type=str, action='store',
        default=cls._DEFAULT_UUID, required=False, help=(
            'The name of the Elasticsearch index. Default: Generate a random '
            'UUID'))

    argument_group.add_argument(
        '--flush_interval', '--flush-interval', dest='flush_interval',
        type=int, action='store', default=cls._DEFAULT_FLUSH_INTERVAL,
        required=False, help=(
            'The number of events to queue up before sent in bulk '
            'to Elasticsearch.'))

    argument_group.add_argument(
        '--doc_type', dest='document_type', type=str,
        action='store', default=cls._DEFAULT_DOCUMENT_TYPE, help=(
            'Name of the document type that will be used in ElasticSearch.'))

    argument_group.add_argument(
        '--username', dest='username', type=str,
        action='store', default=cls._DEFAULT_USERNAME, help=(
            'Username of a Timesketch user that will own the timeline.'))

  # pylint: disable=arguments-differ
  @classmethod
  def ParseOptions(cls, options, output_module):
    """Parses and validates options.

    Args:
      options (argparse.Namespace): parser options.
      output_module (TimesketchOutputModule): output module to configure.

    Raises:
      BadConfigObject: when the output module object is of the wrong type.
      BadConfigOption: when a configuration parameter fails validation.
    """
    if not isinstance(output_module, timesketch_out.TimesketchOutputModule):
      raise errors.BadConfigObject(
          'Output module is not an instance of TimesketchOutputModule')

    document_type = cls._ParseStringOption(
        options, 'document_type', default_value=cls._DEFAULT_DOCUMENT_TYPE)
    output_module.SetDocumentType(document_type)

    flush_interval = cls._ParseNumericOption(
        options, 'flush_interval', default_value=cls._DEFAULT_FLUSH_INTERVAL)
    output_module.SetFlushInterval(flush_interval)

    index = cls._ParseStringOption(
        options, 'index', default_value=cls._DEFAULT_UUID)
    output_module.SetIndexName(index)

    name = cls._ParseStringOption(
        options, 'timeline_name', default_value=cls._DEFAULT_NAME)
    output_module.SetTimelineName(name)

    username = cls._ParseStringOption(
        options, 'username', default_value=cls._DEFAULT_USERNAME)
    output_module.SetTimelineOwner(username)


manager.ArgumentHelperManager.RegisterHelper(TimesketchOutputArgumentsHelper)



# -*- coding: utf-8 -*-
"""Parser for Trend Micro Antivirus logs.

Trend Micro uses two log files to track the scans (both manual/scheduled and
real-time) and the web reputation (network scan/filtering).

Currently only the first log is supported.
"""

from __future__ import unicode_literals

from dfdatetime import definitions as dfdatetime_definitions
from dfdatetime import posix_time as dfdatetime_posix_time
from dfdatetime import time_elements as dfdatetime_time_elements

from plaso.containers import events
from plaso.containers import time_events
from plaso.lib import errors
from plaso.lib import definitions
from plaso.formatters import trendmicroav as formatter
from plaso.parsers import dsv_parser
from plaso.parsers import manager


class TrendMicroAVEventData(events.EventData):
  """Trend Micro AV Log event data.

  Attributes:
    action (str): action.
    threat (str): threat.
    filename (str): filename.
    scan_type (str): scan_type.
  """

  DATA_TYPE = 'av:trendmicro:scan'

  def __init__(self):
    """Initializes event data."""
    super(TrendMicroAVEventData, self).__init__(data_type=self.DATA_TYPE)
    self.threat = None
    self.action = None
    self.path = None
    self.filename = None
    self.scan_type = None


# pylint: disable=abstract-method
class TrendMicroBaseParser(dsv_parser.DSVParser):
  """Common code for parsing Trend Micro log files.

    The file format is reminiscent of CSV, but is not quite the same; the
    delimiter is a three-character sequence and there is no provision for
    quoting or escaping.
  """

  DELIMITER = '<;>'

  # Subclasses must define an integer MIN_COLUMNS value.
  MIN_COLUMNS = None

  def _CreateDictReader(self, parser_mediator, line_reader):
    """Iterates over the log lines and provide a reader for the values.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      line_reader (iter): yields each line in the log file.

    Yields:
      A dictionary of column values keyed by column header.
    """
    for line in line_reader:
      try:
        line = line.decode(self._encoding)
      except UnicodeDecodeError as exception:
        raise errors.UnableToParseFile(
            "Unexpected binary content in file: {0:s}".format(exception))
      stripped_line = line.strip()
      values = stripped_line.split(self.DELIMITER)
      if len(values) < self.MIN_COLUMNS:
        raise errors.UnableToParseFile(
            "Expected at least {0:d} values, found {1:d}".format(
                self.MIN_COLUMNS, len(values)))
      if len(values) > len(self.COLUMNS):
        raise errors.UnableToParseFile(
            "Expected at most {0:d} values, found {1:d}".format(
                len(self.COLUMNS), len(values)))
      yield dict(zip(self.COLUMNS, values))

  def _ParseTimestamp(self, parser_mediator, row):
    """Provides a timestamp for the given row.

    If the Trend Micro log comes from a version that provides a Unix timestamp,
    use that directly; it provides the advantages of UTC and of second
    precision. Otherwise fall back onto the local-timezone date and time.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      row (dict[str, str]): fields of a single row, as specified in COLUMNS.

    Returns:
      dfdatetime.interface.DateTimeValue: the parsed timestamp.
    """
    if 'timestamp' in row:
      try:
        return dfdatetime_posix_time.PosixTime(timestamp=int(row['timestamp']))
      except ValueError as exception:
        parser_mediator.ProduceExtractionError(
            'Log line has a timestamp field: [{0:s}], but it is invalid: {1:s}'
            .format(repr(row['timestamp']), exception))

    # The Unix timestamp is not available; parse the local date and time.
    try:
      return self._ConvertToTimestamp(row['date'], row['time'])
    except ValueError as exception:
      parser_mediator.ProduceExtractionError(
          'Unable to parse time string: [{0:s} {1:s}] with error {2:s}'
          .format(repr(row['date']), repr(row['time']), exception))


  def _ConvertToTimestamp(self, date, time):
    """Converts date and time strings into a timestamp.

    Recent versions of Office Scan write a log field with a Unix timestamp.
    Older versions may not write this field; their logs only provide a date and
    a time expressed in the local time zone. This functions handles the latter
    case.

    Args:
      date (str): date as an 8-character string in the YYYYMMDD format.
      time (str): time as a 3 or 4-character string in the [H]HMM format or a
          6-character string in the HHMMSS format.

    Returns:
      dfdatetime_time_elements.TimestampElements: the parsed timestamp.

    Raises:
      ValueError: if the date/time values cannot be parsed.
    """
    # Check that the strings have the correct length.
    if len(date) != 8:
      raise ValueError('date has wrong length: len({0!s}) != 8'.format(
          repr(date)))
    if len(time) < 3 or len(time) > 4:
      raise ValueError('time has wrong length: len({0!s}) not in (3, 4)'.format(
          repr(time)))

    # Extract the date.
    year = int(date[:4])
    month = int(date[4:6])
    day = int(date[6:8])

    # Extract the time. Note that a single-digit hour value has no leading zero.
    hour = int(time[:-2])
    minutes = int(time[-2:])

    time_elements_tuple = (year, month, day, hour, minutes, 0)
    date_time = dfdatetime_time_elements.TimeElements(
        time_elements_tuple=time_elements_tuple)
    date_time.is_local_time = True
    # TODO: add functionality to dfdatetime to control precision.
    date_time._precision = dfdatetime_definitions.PRECISION_1_MINUTE  # pylint: disable=protected-access

    return date_time

class OfficeScanVirusDetectionParser(TrendMicroBaseParser):
  """Parses the Trend Micro Office Scan Virus Detection Log."""

  NAME = 'trendmicro_vd'
  DESCRIPTION = 'Parser for Trend Micro Office Scan Virus Detection log files.'

  COLUMNS = [
      'date', 'time', 'threat', 'action', 'scan_type', 'unused1',
      'path', 'filename', 'unused2', 'timestamp', 'unused3', 'unused4']
  MIN_COLUMNS = 8

  def __init__(self, *args, **kwargs):
    """Initializes the parser.

    The TrendMicro AV writes a text logfile encoded in the CP1252 charset;
    unless otherwise specified, the parser class needs to know this.
    """
    kwargs.setdefault('encoding', 'cp1252')
    super(OfficeScanVirusDetectionParser, self).__init__(*args, **kwargs)

  def ParseRow(self, parser_mediator, row_offset, row):
    """Parses a line of the log file and produces events.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      row_offset (int): line number of the row.
      row (dict[str, str]): fields of a single row, as specified in COLUMNS.
    """

    timestamp = self._ParseTimestamp(parser_mediator, row)

    if timestamp is None:
      return

    event_data = TrendMicroAVEventData()
    event_data.offset = row_offset
    event_data.threat = row['threat']
    event_data.action = int(row['action'])
    event_data.path = row['path']
    event_data.filename = row['filename']
    event_data.scan_type = int(row['scan_type'])

    event = time_events.DateTimeValuesEvent(
        timestamp, definitions.TIME_DESCRIPTION_WRITTEN)
    parser_mediator.ProduceEventWithEventData(event, event_data)

  def VerifyRow(self, parser_mediator, row):
    """Verifies if a line of the file is in the expected format.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      row (dict[str, str]): fields of a single row, as specified in COLUMNS.

    Returns:
      bool: True if this is the correct parser, False otherwise.
    """
    if len(row) < self.MIN_COLUMNS:
      return False

    # Check the date format!
    # If it doesn't parse, then this isn't a Trend Micro AV log.
    timestamp = self._ConvertToTimestamp(row['date'], row['time'])

    if timestamp is None:
      return False

    # Check that the action value is plausible
    try:
      action = int(row['action'])
    except ValueError:
      return False
    if action not in formatter.SCAN_RESULTS:
      return False

    # All checks passed.
    return True


manager.ParsersManager.RegisterParser(OfficeScanVirusDetectionParser)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests for the tagging file."""

from __future__ import unicode_literals

import unittest

from plaso.engine import tagging_file
from plaso.lib import errors

from tests import test_lib as shared_test_lib


class TaggingFileTestCase(shared_test_lib.BaseTestCase):
  """Tests for the tagging file."""

  # pylint: disable=protected-access

  # TODO: add tests for _ParseDefinitions
  # TODO: add tests for _ParseEventTaggingRule

  @shared_test_lib.skipUnlessHasTestFile([
      'tagging_file', 'invalid_syntax.txt'])
  @shared_test_lib.skipUnlessHasTestFile(['tagging_file', 'valid.txt'])
  def testGetEventTaggingRules(self):
    """Tests the GetEventTaggingRules function."""
    test_path = self._GetTestFilePath(['tagging_file', 'valid.txt'])
    tag_file = tagging_file.TaggingFile(test_path)

    tag_expression = tag_file.GetEventTaggingRules()
    self.assertEqual(len(tag_expression.children), 5)

    test_path = self._GetTestFilePath(['tagging_file', 'invalid_syntax.txt'])
    tag_file = tagging_file.TaggingFile(test_path)

    with self.assertRaises(errors.TaggingFileError):
      tag_file.GetEventTaggingRules()


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests for the tagging analysis plugin."""

from __future__ import unicode_literals

import unittest

from plaso.analysis import tagging
from plaso.lib import timelib
from plaso.containers import events

from tests import test_lib as shared_test_lib
from tests.analysis import test_lib


class TestPrefetchEvent(events.EventObject):
  """A test event type for the tagging analysis plugin."""
  DATA_TYPE = 'windows:prefetch'


class TestChromeDownloadEvent(events.EventObject):
  """A test event type for the tagging analysis plugin."""
  DATA_TYPE = 'chrome:history:file_downloaded'


class TestEvtRecordEvent(events.EventObject):
  """A test event type for the tagging analysis plugin."""
  DATA_TYPE = 'windows:evt:record'


class TaggingAnalysisPluginTest(test_lib.AnalysisPluginTestCase):
  """Tests the tagging analysis plugin."""

  # pylint: disable=protected-access

  _TEST_EVENTS = [
      {'event_type': 'prefetch',
       'timestamp': timelib.Timestamp.CopyFromString('2015-05-01 15:12:00'),
       'attributes': {}
      },
      {'event_type': 'chrome_download',
       'timestamp': timelib.Timestamp.CopyFromString('2015-05-01 05:06:00'),
       'attributes': {}
      },
      {'event_type': 'something_else',
       'timestamp': timelib.Timestamp.CopyFromString('2015-02-19 08:00:01'),
       'attributes': {}
      },
      {'event_type': 'evt',
       'timestamp': timelib.Timestamp.CopyFromString('2016-05-25 13:00:06'),
       'attributes': {
           'source_name': 'Security',
           'event_identifier': 538}
      },
      {'event_type': 'evt',
       'timestamp': timelib.Timestamp.CopyFromString('2016-05-25 13:00:06'),
       'attributes': {
           'source_name': 'Messaging',
           'event_identifier': 16,
           'body': 'this is a message'}
      },
  ]

  # pylint: disable=arguments-differ
  def _CreateTestEventObject(self, event_attributes):
    """Create a test event with a set of attributes.

    Args:
      event_attributes (dict[str, str]): attributes of an event to add to the
          queue.

    Returns:
      EventObject: event with the appropriate attributes for testing.
    """
    if event_attributes['event_type'] == 'prefetch':
      event_object = TestPrefetchEvent()
    elif event_attributes['event_type'] == 'chrome_download':
      event_object = TestChromeDownloadEvent()
    elif event_attributes['event_type'] == 'evt':
      event_object = TestEvtRecordEvent()
    else:
      event_object = events.EventObject()

    event_object.timestamp = event_attributes['timestamp']
    for key, value in iter(event_attributes['attributes'].items()):
      setattr(event_object, key, value)
    return event_object

  @shared_test_lib.skipUnlessHasTestFile(['tagging_file', 'valid.txt'])
  def testExamineEventAndCompileReport(self):
    """Tests the ExamineEvent and CompileReport functions."""
    test_events = []
    for event_dictionary in self._TEST_EVENTS:
      event = self._CreateTestEventObject(event_dictionary)
      test_events.append(event)

    test_file = self._GetTestFilePath(['tagging_file', 'valid.txt'])
    plugin = tagging.TaggingAnalysisPlugin()
    plugin.SetAndLoadTagFile(test_file)

    storage_writer = self._AnalyzeEvents(test_events, plugin)

    self.assertEqual(len(storage_writer.analysis_reports), 1)
    self.assertEqual(storage_writer.number_of_event_tags, 4)

    report = storage_writer.analysis_reports[0]
    self.assertIsNotNone(report)

    expected_text = 'Tagging plugin produced 4 tags.\n'
    self.assertEqual(report.text, expected_text)

    labels = []
    for event_tag in storage_writer.GetEventTags():
      labels.extend(event_tag.labels)

    self.assertEqual(len(labels), 5)

    # This is from a tag rule declared in objectfilter syntax.
    self.assertIn('application_execution', labels)
    # This is from a tag rule declared in dotty syntax.
    self.assertIn('login_attempt', labels)
    # This is from a rule using the "contains" operator
    self.assertIn('text_contains', labels)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests for the plist plugin interface."""

from __future__ import unicode_literals

import unittest

from dfdatetime import posix_time as dfdatetime_posix_time

from plaso.containers import plist_event
from plaso.containers import time_events
from plaso.lib import definitions
from plaso.lib import errors
from plaso.parsers.plist_plugins import interface

from tests.parsers.plist_plugins import test_lib


class MockPlugin(interface.PlistPlugin):
  """Mock plugin."""

  NAME = 'mock_plist_plugin'
  DESCRIPTION = 'Parser for testing parsing plist files.'

  PLIST_PATH = 'plist_binary'
  PLIST_KEYS = frozenset(['DeviceCache', 'PairedDevices'])

  # pylint: disable=arguments-differ
  def GetEntries(self, parser_mediator, **unused_kwargs):
    """Extracts entries for testing.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
    """
    event_data = plist_event.PlistTimeEventData()
    event_data.key = 'LastInquiryUpdate'
    event_data.root = '/DeviceCache/44-00-00-00-00-00'

    date_time = dfdatetime_posix_time.PosixTimeInMicroseconds(
        timestamp=1351827808261762)
    event = time_events.DateTimeValuesEvent(
        date_time, definitions.TIME_DESCRIPTION_WRITTEN)
    parser_mediator.ProduceEventWithEventData(event, event_data)


class TestPlistPlugin(test_lib.PlistPluginTestCase):
  """Tests for the plist plugin interface."""

  # pylint: disable=protected-access

  def setUp(self):
    """Makes preparations before running an individual test."""
    self._top_level_dict = {
        'DeviceCache': {
            '44-00-00-00-00-04': {
                'Name': 'Apple Magic Trackpad 2', 'LMPSubversion': 796,
                'Services': '', 'BatteryPercent': 0.61},
            '44-00-00-00-00-02': {
                'Name': 'test-macpro', 'ClockOffset': 28180,
                'PageScanPeriod': 2, 'PageScanRepetitionMode': 1}}}

  def testGetKeys(self):
    """Tests the _GetKeys function."""
    # Ensure the plugin only processes if both filename and keys exist.
    plugin = MockPlugin()

    # Match DeviceCache from the root level.
    key = ['DeviceCache']
    result = plugin._GetKeys(self._top_level_dict, key)
    self.assertEqual(len(result), 1)

    # Look for a key nested a layer beneath DeviceCache from root level.
    # Note: overriding the default depth to look deeper.
    key = ['44-00-00-00-00-02']
    result = plugin._GetKeys(self._top_level_dict, key, depth=2)
    self.assertEqual(len(result), 1)

    # Check the value of the result was extracted as expected.
    self.assertEqual(result[key[0]]['Name'], 'test-macpro')

  def testProcess(self):
    """Tests the Process function."""
    # Ensure the plugin only processes if both filename and keys exist.
    plugin = MockPlugin()

    # Test correct filename and keys.
    top_level = {'DeviceCache': 1, 'PairedDevices': 1}
    storage_writer = self._ParsePlistWithPlugin(
        plugin, 'plist_binary', top_level)

    self.assertEqual(storage_writer.number_of_events, 1)

    # Correct filename with odd filename cAsinG. Adding an extra useless key.
    top_level = {'DeviceCache': 1, 'PairedDevices': 1, 'R@ndomExtraKey': 1}
    storage_writer = self._ParsePlistWithPlugin(
        plugin, 'pLiSt_BinAry', top_level)

    self.assertEqual(storage_writer.number_of_events, 1)

    # Test wrong filename.
    top_level = {'DeviceCache': 1, 'PairedDevices': 1}
    with self.assertRaises(errors.WrongPlistPlugin):
      _ = self._ParsePlistWithPlugin(
          plugin, 'wrong_file.plist', top_level)

    # Test not enough required keys.
    top_level = {'Useless_Key': 0, 'PairedDevices': 1}
    with self.assertRaises(errors.WrongPlistPlugin):
      _ = self._ParsePlistWithPlugin(
          plugin, 'plist_binary.plist', top_level)

  def testRecurseKey(self):
    """Tests the RecurseKey function."""
    # Ensure with a depth of 1 we only return the root key.
    result = list(interface.RecurseKey(self._top_level_dict, depth=1))
    self.assertEqual(len(result), 1)

    # Trying again with depth limit of 2 this time.
    result = list(interface.RecurseKey(self._top_level_dict, depth=2))
    self.assertEqual(len(result), 3)

    # A depth of two should gives us root plus the two devices. Let's check.
    my_keys = []
    for unused_root, key, unused_value in result:
      my_keys.append(key)
    expected = {'DeviceCache', '44-00-00-00-00-04', '44-00-00-00-00-02'}
    self.assertTrue(expected == set(my_keys))


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests for the Google Chrome history event formatters."""

from __future__ import unicode_literals

import unittest

from plaso.formatters import chrome

from tests.formatters import test_lib


class ChromeFileDownloadFormatterTest(test_lib.EventFormatterTestCase):
  """Tests for the Chrome file download event formatter."""

  def testInitialization(self):
    """Tests the initialization."""
    event_formatter = chrome.ChromeFileDownloadFormatter()
    self.assertIsNotNone(event_formatter)

  def testGetFormatStringAttributeNames(self):
    """Tests the GetFormatStringAttributeNames function."""
    event_formatter = chrome.ChromeFileDownloadFormatter()

    expected_attribute_names = [
        'url',
        'full_path',
        'received_bytes',
        'total_bytes']

    self._TestGetFormatStringAttributeNames(
        event_formatter, expected_attribute_names)

  # TODO: add test for GetMessages.


class ChromePageVisitedFormatterTest(test_lib.EventFormatterTestCase):
  """Tests for the Chrome page visited event formatter."""

  def testInitialization(self):
    """Tests the initialization."""
    event_formatter = chrome.ChromePageVisitedFormatter()
    self.assertIsNotNone(event_formatter)

  def testGetFormatStringAttributeNames(self):
    """Tests the GetFormatStringAttributeNames function."""
    event_formatter = chrome.ChromePageVisitedFormatter()

    expected_attribute_names = [
        'url',
        'title',
        'typed_count',
        'from_visit',
        'visit_source',
        'page_transition',
        'extra']

    self._TestGetFormatStringAttributeNames(
        event_formatter, expected_attribute_names)

  # TODO: add test for GetMessages.


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests for the nsrlsvr analysis plugin CLI arguments helper."""

from __future__ import unicode_literals

import argparse
import unittest

from plaso.analysis import nsrlsvr
from plaso.lib import errors
from plaso.cli.helpers import nsrlsvr_analysis

from tests.cli import test_lib as cli_test_lib
from tests.cli.helpers import test_lib


class NsrlsvrAnalysisArgumentsHelperTest(
    test_lib.AnalysisPluginArgumentsHelperTest):
  """Tests the nsrlsvr analysis plugin CLI arguments helper."""

  # pylint: disable=no-member,protected-access

  _EXPECTED_OUTPUT = """\
usage: cli_helper.py [--nsrlsvr-hash HASH] [--nsrlsvr-host HOST]
                     [--nsrlsvr-label LABEL] [--nsrlsvr-port PORT]

Test argument parser.

optional arguments:
  --nsrlsvr-hash HASH, --nsrlsvr_hash HASH
                        Type of hash to use to query nsrlsvr instance, the
                        default is: md5. Supported options: md5, sha1
  --nsrlsvr-host HOST, --nsrlsvr_host HOST
                        Hostname or IP address of the nsrlsvr instance to
                        query, the default is: localhost
  --nsrlsvr-label LABEL, --nsrlsvr_label LABEL
                        Label to apply to events, the default is:
                        nsrl_present.
  --nsrlsvr-port PORT, --nsrlsvr_port PORT
                        Port number of the nsrlsvr instance to query, the
                        default is: 9120.
"""

  def testAddArguments(self):
    """Tests the AddArguments function."""
    argument_parser = argparse.ArgumentParser(
        prog='cli_helper.py',
        description='Test argument parser.', add_help=False,
        formatter_class=cli_test_lib.SortedArgumentsHelpFormatter)

    nsrlsvr_analysis.NsrlsvrAnalysisArgumentsHelper.AddArguments(
        argument_parser)

    output = self._RunArgparseFormatHelp(argument_parser)
    self.assertEqual(output, self._EXPECTED_OUTPUT)

  def testParseOptions(self):
    """Tests the ParseOptions function."""
    options = cli_test_lib.TestOptions()
    analysis_plugin = nsrlsvr.NsrlsvrAnalysisPlugin()

    options.nsrlsvr_hash = 'sha1'
    options.nsrlsvr_host = '127.0.0.1'
    options.nsrlsvr_port = 9120
    options.nsrlsvr_label = 'NSRLSVR'

    with self.assertRaises(errors.BadConfigOption):
      nsrlsvr_analysis.NsrlsvrAnalysisArgumentsHelper.ParseOptions(
          options, analysis_plugin)

    self.assertEqual(analysis_plugin._analyzer._host, '127.0.0.1')
    self.assertEqual(analysis_plugin._analyzer._port, 9120)
    self.assertEqual(analysis_plugin._label, 'NSRLSVR')

    with self.assertRaises(errors.BadConfigObject):
      nsrlsvr_analysis.NsrlsvrAnalysisArgumentsHelper.ParseOptions(
          options, None)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Parser for Android WebviewCache databases."""

from __future__ import unicode_literals

from dfdatetime import java_time as dfdatetime_java_time

from plaso.containers import events
from plaso.containers import time_events
from plaso.lib import definitions
from plaso.parsers import sqlite
from plaso.parsers.sqlite_plugins import interface


class AndroidWebViewCacheEventData(events.EventData):
  """Android WebViewCache event data.

  Attributes:
    content_length (int): size of the cached content.
    url (str): URL the content was retrieved from.
  """

  DATA_TYPE = 'android:webviewcache'

  def __init__(self):
    """Initializes event data."""
    super(AndroidWebViewCacheEventData, self).__init__(data_type=self.DATA_TYPE)
    self.content_length = None
    self.url = None


class AndroidWebViewCachePlugin(interface.SQLitePlugin):
  """Parser for Android WebViewCache databases."""

  NAME = 'android_webviewcache'
  DESCRIPTION = 'Parser for Android WebViewCache databases'

  REQUIRED_TABLES = frozenset(['android_metadata', 'cache'])

  QUERIES = frozenset([
      ('SELECT url, contentlength, expires, lastmodify FROM cache',
       'ParseRow')])

  SCHEMAS = [{
      'android_metadata': (
          'CREATE TABLE android_metadata (locale TEXT)'),
      'cache': (
          'CREATE TABLE cache (_id INTEGER PRIMARY KEY, url TEXT, filepath '
          'TEXT, lastmodify TEXT, etag TEXT, expires INTEGER, expiresstring '
          'TEXT, mimetype TEXT, encoding TEXT, httpstatus INTEGER, location '
          'TEXT, contentlength INTEGER, contentdisposition TEXT, UNIQUE (url) '
          'ON CONFLICT REPLACE)')}]

  def ParseRow(self, parser_mediator, query, row, **unused_kwargs):
    """Parses a row from the database.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      query (str): query that created the row.
      row (sqlite3.Row): row.
    """
    query_hash = hash(query)

    event_data = AndroidWebViewCacheEventData()
    event_data.content_length = self._GetRowValue(
        query_hash, row, 'contentlength')
    event_data.query = query
    event_data.url = self._GetRowValue(query_hash, row, 'url')

    timestamp = self._GetRowValue(query_hash, row, 'expires')
    if timestamp is not None:
      date_time = dfdatetime_java_time.JavaTime(timestamp=timestamp)
      event = time_events.DateTimeValuesEvent(
          date_time, definitions.TIME_DESCRIPTION_EXPIRATION)
      parser_mediator.ProduceEventWithEventData(event, event_data)

    timestamp = self._GetRowValue(query_hash, row, 'lastmodify')
    if timestamp is not None:
      date_time = dfdatetime_java_time.JavaTime(timestamp=timestamp)
      event = time_events.DateTimeValuesEvent(
          date_time, definitions.TIME_DESCRIPTION_MODIFICATION)
      parser_mediator.ProduceEventWithEventData(event, event_data)


sqlite.SQLiteParser.RegisterPlugin(AndroidWebViewCachePlugin)



# -*- coding: utf-8 -*-
"""The Kik kik.sqlite iOS database event formatter."""

from __future__ import unicode_literals

from plaso.formatters import interface
from plaso.formatters import manager
from plaso.lib import errors

class KikIOSMessageFormatter(interface.ConditionalEventFormatter):
  """Formatter for an iOS Kik message event."""

  DATA_TYPE = 'ios:kik:messaging'

  FORMAT_STRING_PIECES = [
      'Username: {username}',
      'Displayname: {displayname}',
      'Status: {message_status}',
      'Type: {message_type}',
      'Message: {body}']

  FORMAT_STRING_SHORT_PIECES = ['{body}']

  SOURCE_LONG = 'Kik iOS messages'
  SOURCE_SHORT = 'Kik iOS'

  _MESSAGE_TYPE = {
      1: 'received',
      2: 'sent',
      3: 'message to group admin',
      4: 'message to group'
  }

  _MESSAGE_STATUS = {
      0: 'unread',
      2: 'message not sent',
      6: 'sent to Kik server',
      14: 'delivered ',
      16: 'message read',
      30: 'read after immediate delivery',
      38: 'not a member of the group',
      66: 'delay in reaching Kik server',
      70: 'push notification sent',
      78: 'delivered after offline',
      94: 'read after offline'
  }

  def GetMessages(self, unused_formatter_mediator, event):
    """Determines the formatted message strings for an event object.

    Args:
      formatter_mediator (FormatterMediator): mediates the interactions between
          formatters and other components, such as storage and Windows EventLog
          resources.
      event (EventObject): event.

    Returns:
      tuple(str, str): formatted message string and short message string.

    Raises:
      WrongFormatter: if the event object cannot be formatted by the formatter.
    """
    if self.DATA_TYPE != event.data_type:
      raise errors.WrongFormatter('Unsupported data type: {0:s}.'.format(
          event.data_type))

    event_values = event.CopyToDict()

    message_type = event_values.get('message_type', None)
    if message_type is not None:
      event_values['message_type'] = (
          self._MESSAGE_TYPE.get(message_type, 'UNKNOWN'))

    message_status = event_values.get('message_status', None)
    if message_status is not None:
      event_values['message_status'] = (
          self._MESSAGE_STATUS.get(message_status, 'UNKNOWN'))

    return self._ConditionalFormatMessages(event_values)


manager.FormattersManager.RegisterFormatter(KikIOSMessageFormatter)



# -*- coding: utf-8 -*-
"""This file contains the analysis plugin manager class."""

from __future__ import unicode_literals

from plaso.analysis import definitions


class AnalysisPluginManager(object):
  """Analysis plugin manager."""

  _plugin_classes = {}

  _PLUGIN_TYPE_STRINGS = {
      definitions.PLUGIN_TYPE_ANNOTATION: (
          'Annotation/Tagging plugin'),
      definitions.PLUGIN_TYPE_ANOMALY: (
          'Anomaly plugin'),
      definitions.PLUGIN_TYPE_REPORT: (
          'Summary/Report plugin'),
      definitions.PLUGIN_TYPE_STATISTICS: (
          'Statistics plugin')
  }
  _PLUGIN_TYPE_STRINGS.setdefault('Unknown type')

  @classmethod
  def DeregisterPlugin(cls, plugin_class):
    """Deregisters an analysis plugin class.

    The analysis plugin classes are identified by their lower case name.

    Args:
      plugin_class (type): class of the analysis plugin.

    Raises:
      KeyError: if an analysis plugin class is not set for the corresponding
          name.
    """
    plugin_name = plugin_class.NAME.lower()
    if plugin_name not in cls._plugin_classes:
      raise KeyError('Plugin class not set for name: {0:s}.'.format(
          plugin_class.NAME))

    del cls._plugin_classes[plugin_name]

  # TODO: refactor to match parsers manager.
  @classmethod
  def GetAllPluginInformation(cls, show_all=True):
    """Retrieves a list of the registered analysis plugins.

    Args:
      show_all (Optional[bool]): True if all analysis plugin names should
          be listed.

    Returns:
      list[tuple[str, str, str]]: the name, docstring and type string of each
          analysis plugin in alphabetical order.
    """
    results = []
    for plugin_class in iter(cls._plugin_classes.values()):
      plugin_object = plugin_class()
      if not show_all and not plugin_class.ENABLE_IN_EXTRACTION:
        continue

      # TODO: Use a specific description variable, not the docstring.
      doc_string, _, _ = plugin_class.__doc__.partition('\n')
      type_string = cls._PLUGIN_TYPE_STRINGS.get(plugin_object.plugin_type)
      information_tuple = (plugin_object.plugin_name, doc_string, type_string)
      results.append(information_tuple)

    return sorted(results)

  @classmethod
  def GetPluginNames(cls):
    """Retrieves the analysis  plugin names.

    Returns:
      list[str]: analysis plugin names.
    """
    return sorted(cls._plugin_classes.keys())

  @classmethod
  def GetPluginObjects(cls, plugin_names):
    """Retrieves the plugin objects.

    Args:
      plugin_names (list[str]): names of plugins that should be retrieved.

    Returns:
      dict[str, AnalysisPlugin]: analysis plugins per name.
    """
    plugin_objects = {}
    for plugin_name, plugin_class in iter(cls._plugin_classes.items()):
      if plugin_name not in plugin_names:
        continue

      plugin_objects[plugin_name] = plugin_class()

    return plugin_objects

  @classmethod
  def GetPlugins(cls):
    """Retrieves the registered analysis plugin classes.

    Yields:
      tuple: contains:

        str: name of the plugin
        type: plugin class
    """
    for plugin_name, plugin_class in iter(cls._plugin_classes.items()):
      yield plugin_name, plugin_class

  @classmethod
  def RegisterPlugin(cls, plugin_class):
    """Registers an analysis plugin class.

    Then analysis plugin classes are identified based on their lower case name.

    Args:
      plugin_class (type): class of the analysis plugin.

    Raises:
      KeyError: if an analysis plugin class is already set for the corresponding
          name.
    """
    plugin_name = plugin_class.NAME.lower()
    if plugin_name in cls._plugin_classes:
      raise KeyError('Plugin class already set for name: {0:s}.'.format(
          plugin_class.NAME))

    cls._plugin_classes[plugin_name] = plugin_class

  @classmethod
  def RegisterPlugins(cls, plugin_classes):
    """Registers analysis plugin classes.

    The analysis plugin classes are identified based on their lower case name.

    Args:
      plugin_classes (list[type]): classes of the analysis plugin.

    Raises:
      KeyError: if an analysis plugin class is already set for the corresponding
          name.
    """
    for plugin_class in plugin_classes:
      cls.RegisterPlugin(plugin_class)



# -*- coding: utf-8 -*-
"""This file contains classes used for preprocessing in plaso."""

from __future__ import unicode_literals

import abc

from artifacts import definitions as artifact_definitions
from dfvfs.helpers import file_system_searcher
from dfwinreg import registry_searcher

from plaso.lib import errors


class ArtifactPreprocessorPlugin(object):
  """The artifact preprocessor plugin interface.

  The artifact preprocessor determines preprocessing attributes based on
  an artifact definition defined by ARTIFACT_DEFINITION_NAME.
  """

  ARTIFACT_DEFINITION_NAME = None


class FileSystemArtifactPreprocessorPlugin(ArtifactPreprocessorPlugin):
  """File system artifact preprocessor plugin interface.

  Shared functionality for preprocessing attributes based on a file system
  artifact definition, such as file or path.
  """

  @abc.abstractmethod
  def _ParsePathSpecification(
      self, knowledge_base, searcher, file_system, path_specification,
      path_separator):
    """Parses a file system for a preprocessing attribute.

    Args:
      knowledge_base (KnowledgeBase): to fill with preprocessing information.
      searcher (dfvfs.FileSystemSearcher): file system searcher to preprocess
          the file system.
      file_system (dfvfs.FileSystem): file system to be preprocessed.
      path_specification (dfvfs.PathSpec): path specification that contains
          the artifact value data.
      path_separator (str): path segment separator.

    Raises:
      errors.PreProcessFail: if the preprocessing fails.
    """

  def Collect(
      self, knowledge_base, artifact_definition, searcher, file_system):
    """Collects values using a file artifact definition.

    Args:
      knowledge_base (KnowledgeBase): to fill with preprocessing information.
      artifact_definition (artifacts.ArtifactDefinition): artifact definition.
      searcher (dfvfs.FileSystemSearcher): file system searcher to preprocess
          the file system.
      file_system (dfvfs.FileSystem): file system to be preprocessed.

    Raises:
      PreProcessFail: if the Windows Registry key or value cannot be read.
    """
    for source in artifact_definition.sources:
      if source.type_indicator not in (
          artifact_definitions.TYPE_INDICATOR_FILE,
          artifact_definitions.TYPE_INDICATOR_PATH):
        continue

      for path in source.paths:
        # Make sure the path separators used in the artifact definition
        # correspond to those used by the file system.
        path_segments = path.split(source.separator)

        find_spec = file_system_searcher.FindSpec(
            location_glob=path_segments[1:], case_sensitive=False)

        for path_specification in searcher.Find(find_specs=[find_spec]):
          self._ParsePathSpecification(
              knowledge_base, searcher, file_system, path_specification,
              source.separator)


class FileEntryArtifactPreprocessorPlugin(FileSystemArtifactPreprocessorPlugin):
  """File entry artifact preprocessor plugin interface.

  Shared functionality for preprocessing attributes based on a file entry
  artifact definition, such as file or path.
  """

  @abc.abstractmethod
  def _ParseFileEntry(self, knowledge_base, file_entry):
    """Parses a file entry for a preprocessing attribute.

    Args:
      knowledge_base (KnowledgeBase): to fill with preprocessing information.
      file_entry (dfvfs.FileEntry): file entry that contains the artifact
          value data.

    Raises:
      errors.PreProcessFail: if the preprocessing fails.
    """

  def _ParsePathSpecification(
      self, knowledge_base, searcher, file_system, path_specification,
      path_separator):
    """Parses a file system for a preprocessing attribute.

    Args:
      knowledge_base (KnowledgeBase): to fill with preprocessing information.
      searcher (dfvfs.FileSystemSearcher): file system searcher to preprocess
          the file system.
      file_system (dfvfs.FileSystem): file system to be preprocessed.
      path_specification (dfvfs.PathSpec): path specification that contains
          the artifact value data.
      path_separator (str): path segment separator.

    Raises:
      errors.PreProcessFail: if the preprocessing fails.
    """
    try:
      file_entry = searcher.GetFileEntryByPathSpec(path_specification)
    except IOError as exception:
      relative_path = searcher.GetRelativePath(path_specification)
      if path_separator != file_system.PATH_SEPARATOR:
        relative_path_segments = file_system.SplitPath(relative_path)
        relative_path = '{0:s}{1:s}'.format(
            path_separator, path_separator.join(relative_path_segments))

      raise errors.PreProcessFail((
          'Unable to retrieve file entry: {0:s} with error: '
          '{1:s}').format(relative_path, exception))

    self._ParseFileEntry(knowledge_base, file_entry)


class FileArtifactPreprocessorPlugin(FileEntryArtifactPreprocessorPlugin):
  """File artifact preprocessor plugin interface.

  Shared functionality for preprocessing attributes based on a file artifact
  definition, such as file or path.
  """

  @abc.abstractmethod
  def _ParseFileData(self, knowledge_base, file_object):
    """Parses file content (data) for a preprocessing attribute.

    Args:
      knowledge_base (KnowledgeBase): to fill with preprocessing information.
      file_object (dfvfs.FileIO): file-like object that contains the artifact
          value data.

    Raises:
      errors.PreProcessFail: if the preprocessing fails.
    """

  def _ParseFileEntry(self, knowledge_base, file_entry):
    """Parses a file entry for a preprocessing attribute.

    Args:
      knowledge_base (KnowledgeBase): to fill with preprocessing information.
      file_entry (dfvfs.FileEntry): file entry that contains the artifact
          value data.

    Raises:
      errors.PreProcessFail: if the preprocessing fails.
    """
    file_object = file_entry.GetFileObject()
    try:
      self._ParseFileData(knowledge_base, file_object)
    finally:
      file_object.close()


class WindowsRegistryKeyArtifactPreprocessorPlugin(ArtifactPreprocessorPlugin):
  """Windows Registry key artifact preprocessor plugin interface.

  Shared functionality for preprocessing attributes based on a Windows
  Registry artifact definition, such as Windows Registry key or value.
  """

  @abc.abstractmethod
  def _ParseKey(self, knowledge_base, registry_key, value_name):
    """Parses a Windows Registry key for a preprocessing attribute.

    Args:
      knowledge_base (KnowledgeBase): to fill with preprocessing information.
      registry_key (dfwinreg.WinRegistryKey): Windows Registry key.
      value_name (str): name of the Windows Registry value.

    Raises:
      errors.PreProcessFail: if the preprocessing fails.
    """

  def Collect(
      self, knowledge_base, artifact_definition, searcher):
    """Collects values using a Windows Registry value artifact definition.

    Args:
      knowledge_base (KnowledgeBase): to fill with preprocessing information.
      artifact_definition (artifacts.ArtifactDefinition): artifact definition.
      searcher (dfwinreg.WinRegistrySearcher): Windows Registry searcher to
          preprocess the Windows Registry.

    Raises:
      PreProcessFail: if the Windows Registry key or value cannot be read.
    """
    for source in artifact_definition.sources:
      if source.type_indicator not in (
          artifact_definitions.TYPE_INDICATOR_WINDOWS_REGISTRY_KEY,
          artifact_definitions.TYPE_INDICATOR_WINDOWS_REGISTRY_VALUE):
        continue

      if source.type_indicator == (
          artifact_definitions.TYPE_INDICATOR_WINDOWS_REGISTRY_KEY):
        key_value_pairs = [{'key': key} for key in source.keys]
      else:
        key_value_pairs = source.key_value_pairs

      for key_value_pair in key_value_pairs:
        key_path = key_value_pair['key']

        # The artifact definitions currently incorrectly define
        # CurrentControlSet so we correct it here for now.
        # Also see: https://github.com/ForensicArtifacts/artifacts/issues/120
        key_path_upper = key_path.upper()
        if key_path_upper.startswith('%%CURRENT_CONTROL_SET%%'):
          key_path = '{0:s}{1:s}'.format(
              'HKEY_LOCAL_MACHINE\\System\\CurrentControlSet', key_path[23:])

        find_spec = registry_searcher.FindSpec(key_path_glob=key_path)

        for key_path in searcher.Find(find_specs=[find_spec]):
          try:
            registry_key = searcher.GetKeyByPath(key_path)
          except IOError as exception:
            raise errors.PreProcessFail((
                'Unable to retrieve Windows Registry key: {0:s} with error: '
                '{1:s}').format(key_path, exception))

          if registry_key:
            value_name = key_value_pair.get('value', None)
            self._ParseKey(knowledge_base, registry_key, value_name)


class WindowsRegistryValueArtifactPreprocessorPlugin(
    WindowsRegistryKeyArtifactPreprocessorPlugin):
  """Windows Registry value artifact preprocessor plugin interface.

  Shared functionality for preprocessing attributes based on a Windows
  Registry value artifact definition.
  """

  def _ParseKey(self, knowledge_base, registry_key, value_name):
    """Parses a Windows Registry key for a preprocessing attribute.

    Args:
      knowledge_base (KnowledgeBase): to fill with preprocessing information.
      registry_key (dfwinreg.WinRegistryKey): Windows Registry key.
      value_name (str): name of the Windows Registry value.

    Raises:
      errors.PreProcessFail: if the preprocessing fails.
    """
    try:
      registry_value = registry_key.GetValueByName(value_name)
    except IOError as exception:
      raise errors.PreProcessFail((
          'Unable to retrieve Windows Registry key: {0:s} value: {1:s} '
          'with error: {2!s}').format(
              registry_key.path, value_name, exception))

    if registry_value:
      value_object = registry_value.GetDataAsObject()
      if value_object:
        self._ParseValueData(knowledge_base, value_object)

  @abc.abstractmethod
  def _ParseValueData(self, knowledge_base, value_data):
    """Parses Windows Registry value data for a preprocessing attribute.

    Args:
      knowledge_base (KnowledgeBase): to fill with preprocessing information.
      value_data (object): Windows Registry value data.

    Raises:
      errors.PreProcessFail: if the preprocessing fails.
    """



# -*- coding: utf-8 -*-
"""This file contains the Basic Security Module definitions."""

from __future__ import unicode_literals


# Arbitrary tokens.
# Type of data to print in a BSM_TOKEN_DATA.
BSM_TOKEN_DATA_TYPE = {
    0: 'AUR_CHAR',
    1: 'AUR_SHORT',
    2: 'AUR_INT32'}

BSM_TOKEN_DATA_PRINT = {
    0: 'Binary',
    1: 'Octal',
    2: 'Decimal',
    3: 'Hexadecimal',
    4: 'String'}

# BSM identification errors.
BSM_ERRORS = {
    0: 'Success',
    1: 'Operation not permitted',
    2: 'No such file or directory',
    3: 'No such process',
    4: 'Interrupted system call',
    5: 'Input/output error',
    6: 'Device not configured',
    7: 'Argument list too long',
    8: 'Exec format error',
    9: 'Bad file descriptor',
    10: 'No child processes',
    11: 'Resource temporarily unavailable',
    12: 'Cannot allocate memory',
    13: 'Permission denied',
    14: 'Bad address',
    15: 'Block device required',
    16: 'Device busy',
    17: 'File exists',
    18: 'ross-device link',
    19: 'Operation not supported by device',
    20: 'Not a directory',
    21: 'Is a directory',
    22: 'Invalid argument',
    23: 'Too many open files in system',
    24: 'Too many open files',
    25: 'Inappropriate ioctl for device',
    26: 'Text file busy',
    27: 'File too large',
    28: 'No space left on device',
    29: 'Illegal seek',
    30: 'Read-only file system',
    31: 'Too many links',
    32: 'Broken pipe',
    33: 'Numerical argument out of domain',
    34: 'Result too large',
    35: 'No message of desired type',
    36: 'Identifier removed',
    45: 'Resource deadlock avoided',
    46: 'No locks available',
    47: 'Operation canceled',
    48: 'Operation not supported',
    49: 'Disc quota exceeded',
    66: 'Too many levels of remote in path',
    67: 'Link has been severed',
    71: 'Protocol error',
    74: 'Multihop attempted',
    77: 'Bad message',
    78: 'File name too long',
    79: 'Value too large to be stored in data type',
    88: 'Illegal byte sequence',
    89: 'Function not implemented',
    90: 'Too many levels of symbolic links',
    91: 'Restart syscall',
    93: 'Directory not empty',
    94: 'Too many users',
    95: 'Socket operation on non-socket',
    96: 'Destination address required',
    97: 'Message too long',
    98: 'Protocol wrong type for socket',
    99: 'Protocol not available',
    120: 'Protocol not supported',
    121: 'Socket type not supported',
    122: 'Operation not supported',
    123: 'Protocol family not supported',
    124: 'Address family not supported by protocol family',
    125: 'Address already in use',
    126: 'Can\'t assign requested address',
    127: 'Network is down',
    128: 'Network unreachable',
    129: 'Network dropped connection on reset',
    130: 'Software caused connection abort',
    131: 'Connection reset by peer',
    132: 'No buffer space available',
    133: 'Socket is already connected',
    134: 'Socket is not connected',
    143: 'Can\'t send after socket shutdown',
    144: 'Too many references: can\'t splice',
    145: 'Operation timed out',
    146: 'Connection refused',
    147: 'Host is down',
    148: 'No route to host',
    149: 'Operation already in progress',
    150: 'Operation now in progress',
    151: 'Stale NFS file handle',
    190: 'PROCLIM',
    191: 'BADRPC',
    192: 'RPCMISMATCH',
    193: 'PROGUNAVAIL',
    194: 'PROGMISMATCH',
    195: 'PROCUNAVAIL',
    196: 'FTYPE',
    197: 'AUTH',
    198: 'NEEDAUTH',
    199: 'NOATTR',
    200: 'DOOFUS',
    201: 'USTRETURN',
    202: 'NOIOCTL',
    203: 'DIRIOCTL',
    204: 'PWROFF',
    205: 'DEVERR',
    206: 'BADEXEC',
    207: 'BADARCH',
    208: 'SHLIBVERS',
    209: 'BADMACHO',
    210: 'POLICY'}

# BSM network protocols. The information comes from OpenBSD project,
# so it might not be directly applicable to MacOS.
BSM_PROTOCOLS = {
    0: 'UNSPEC',
    1: 'LOCAL',
    2: 'INET',
    3: 'IMPLINK',
    4: 'PUP',
    5: 'CHAOS',
    6: 'NS',
    8: 'ECMA',
    9: 'DATAKIT',
    10: 'CCITT',
    11: 'SNA',
    12: 'DECnet',
    13: 'DLI',
    14: 'LAT',
    15: 'HYLINK',
    16: 'APPLETALK',
    19: 'OSI',
    23: 'IPX',
    24: 'ROUTE',
    25: 'LINK',
    26: 'INET6',
    27: 'KEY',
    500: 'NETBIOS',
    501: 'ISO',
    502: 'XTP',
    503: 'COIP',
    504: 'CNT',
    505: 'RTIP',
    506: 'SIP',
    507: 'PIP',
    508: 'ISDN',
    509: 'E164',
    510: 'NATM',
    511: 'ATM',
    512: 'NETGRAPH',
    513: 'SLOW',
    514: 'CLUSTER',
    515: 'ARP',
    516: 'BLUETOOTH'}

# key event types.
BSM_AUDIT_EVENT = {
    0: 'indir system call',
    1: 'exit(2)',
    2: 'fork(2)',
    3: 'open(2) - attr only',
    4: 'creat(2)',
    5: 'link(2)',
    6: 'unlink(2)',
    7: 'exec(2)',
    8: 'chdir(2)',
    9: 'mknod(2)',
    10: 'chmod(2)',
    11: 'chown(2)',
    12: 'umount(2) - old version',
    13: 'junk',
    14: 'access(2)',
    15: 'kill(2)',
    16: 'stat(2)',
    17: 'lstat(2)',
    18: 'acct(2)',
    19: 'mctl(2)',
    20: 'reboot(2)',
    21: 'symlink(2)',
    22: 'readlink(2)',
    23: 'execve(2)',
    24: 'chroot(2)',
    25: 'vfork(2)',
    26: 'setgroups(2)',
    27: 'setpgrp(2)',
    28: 'swapon(2)',
    29: 'sethostname(2)',
    30: 'fcntl(2)',
    31: 'setpriority(2)',
    32: 'connect(2)',
    33: 'accept(2)',
    34: 'bind(2)',
    35: 'setsockopt(2)',
    36: 'vtrace(2)',
    37: 'settimeofday(2)',
    38: 'fchown(2)',
    39: 'fchmod(2)',
    40: 'setreuid(2)',
    41: 'setregid(2)',
    42: 'rename(2)',
    43: 'truncate(2)',
    44: 'ftruncate(2)',
    45: 'flock(2)',
    46: 'shutdown(2)',
    47: 'mkdir(2)',
    48: 'rmdir(2)',
    49: 'utimes(2)',
    50: 'adjtime(2)',
    51: 'setrlimit(2)',
    52: 'killpg(2)',
    53: 'nfs_svc(2)',
    54: 'statfs(2)',
    55: 'fstatfs(2)',
    56: 'unmount(2)',
    57: 'async_daemon(2)',
    58: 'nfs_getfh(2)',
    59: 'setdomainname(2)',
    60: 'quotactl(2)',
    61: 'exportfs(2)',
    62: 'mount(2)',
    63: 'semsys(2)',
    64: 'msgsys(2)',
    65: 'shmsys(2)',
    66: 'bsmsys(2)',
    67: 'rfssys(2)',
    68: 'fchdir(2)',
    69: 'fchroot(2)',
    70: 'vpixsys(2)',
    71: 'pathconf(2)',
    72: 'open(2) - read',
    73: 'open(2) - read,creat',
    74: 'open(2) - read,trunc',
    75: 'open(2) - read,creat,trunc',
    76: 'open(2) - write',
    77: 'open(2) - write,creat',
    78: 'open(2) - write,trunc',
    79: 'open(2) - write,creat,trunc',
    80: 'open(2) - read,write',
    81: 'open(2) - read,write,creat',
    82: 'open(2) - read,write,trunc',
    83: 'open(2) - read,write,creat,trunc',
    84: 'msgctl(2) - illegal command',
    85: 'msgctl(2) - IPC_RMID command',
    86: 'msgctl(2) - IPC_SET command',
    87: 'msgctl(2) - IPC_STAT command',
    88: 'msgget(2)',
    89: 'msgrcv(2)',
    90: 'msgsnd(2)',
    91: 'shmctl(2) - illegal command',
    92: 'shmctl(2) - IPC_RMID command',
    93: 'shmctl(2) - IPC_SET command',
    94: 'shmctl(2) - IPC_STAT command',
    95: 'shmget(2)',
    96: 'shmat(2)',
    97: 'shmdt(2)',
    98: 'semctl(2) - illegal command',
    99: 'semctl(2) - IPC_RMID command',
    100: 'semctl(2) - IPC_SET command',
    101: 'semctl(2) - IPC_STAT command',
    102: 'semctl(2) - GETNCNT command',
    103: 'semctl(2) - GETPID command',
    104: 'semctl(2) - GETVAL command',
    105: 'semctl(2) - GETALL command',
    106: 'semctl(2) - GETZCNT command',
    107: 'semctl(2) - SETVAL command',
    108: 'semctl(2) - SETALL command',
    109: 'semget(2)',
    110: 'semop(2)',
    111: 'process dumped core',
    112: 'close(2)',
    113: 'system booted',
    114: 'async_daemon(2) exited',
    115: 'nfssvc(2) exited',
    128: 'writel(2)',
    129: 'writevl(2)',
    130: 'getauid(2)',
    131: 'setauid(2)',
    132: 'getaudit(2)',
    133: 'setaudit(2)',
    134: 'getuseraudit(2)',
    135: 'setuseraudit(2)',
    136: 'auditsvc(2)',
    137: 'audituser(2)',
    138: 'auditon(2)',
    139: 'auditon(2) - GETTERMID command',
    140: 'auditon(2) - SETTERMID command',
    141: 'auditon(2) - GPOLICY command',
    142: 'auditon(2) - SPOLICY command',
    143: 'auditon(2) - GESTATE command',
    144: 'auditon(2) - SESTATE command',
    145: 'auditon(2) - GQCTRL command',
    146: 'auditon(2) - SQCTRL command',
    147: 'getkernstate(2)',
    148: 'setkernstate(2)',
    149: 'getportaudit(2)',
    150: 'auditstat(2)',
    151: 'revoke(2)',
    152: 'Solaris AUE_MAC',
    153: 'enter prom',
    154: 'exit prom',
    155: 'Solaris AUE_IFLOAT',
    156: 'Solaris AUE_PFLOAT',
    157: 'Solaris AUE_UPRIV',
    158: 'ioctl(2)',
    173: 'one-sided session record',
    174: 'msggetl(2)',
    175: 'msgrcvl(2)',
    176: 'msgsndl(2)',
    177: 'semgetl(2)',
    178: 'shmgetl(2)',
    183: 'socket(2)',
    184: 'sendto(2)',
    185: 'pipe(2)',
    186: 'socketpair(2)',
    187: 'send(2)',
    188: 'sendmsg(2)',
    189: 'recv(2)',
    190: 'recvmsg(2)',
    191: 'recvfrom(2)',
    192: 'read(2)',
    193: 'getdents(2)',
    194: 'lseek(2)',
    195: 'write(2)',
    196: 'writev(2)',
    197: 'nfs server',
    198: 'readv(2)',
    199: 'Solaris old stat(2)',
    200: 'setuid(2)',
    201: 'old stime(2)',
    202: 'old utime(2)',
    203: 'old nice(2)',
    204: 'Solaris old setpgrp(2)',
    205: 'setgid(2)',
    206: 'readl(2)',
    207: 'readvl(2)',
    208: 'fstat(2)',
    209: 'dup2(2)',
    210: 'mmap(2)',
    211: 'audit(2)',
    212: 'Solaris priocntlsys(2)',
    213: 'munmap(2)',
    214: 'setegid(2)',
    215: 'seteuid(2)',
    216: 'putmsg(2)',
    217: 'getmsg(2)',
    218: 'putpmsg(2)',
    219: 'getpmsg(2)',
    220: 'audit system calls place holder',
    221: 'auditon(2) - get kernel mask',
    222: 'auditon(2) - set kernel mask',
    223: 'auditon(2) - get cwd',
    224: 'auditon(2) - get car',
    225: 'auditon(2) - get audit statistics',
    226: 'auditon(2) - reset audit statistics',
    227: 'auditon(2) - set mask per uid',
    228: 'auditon(2) - set mask per session ID',
    229: 'auditon(2) - get audit state',
    230: 'auditon(2) - set audit state',
    231: 'auditon(2) - get event class',
    232: 'auditon(2) - set event class',
    233: 'utssys(2) - fusers',
    234: 'statvfs(2)',
    235: 'xstat(2)',
    236: 'lxstat(2)',
    237: 'lchown(2)',
    238: 'memcntl(2)',
    239: 'sysinfo(2)',
    240: 'xmknod(2)',
    241: 'fork1(2)',
    242: 'modctl(2) system call place holder',
    243: 'modctl(2) - load module',
    244: 'modctl(2) - unload module',
    245: 'modctl(2) - configure module',
    246: 'modctl(2) - bind module',
    247: 'getmsg-accept',
    248: 'putmsg-connect',
    249: 'putmsg-send',
    250: 'getmsg-receive',
    251: 'acl(2) - SETACL comand',
    252: 'facl(2) - SETACL command',
    253: 'doorfs(2) - system call place holder',
    254: 'doorfs(2) - DOOR_CALL',
    255: 'doorfs(2) - DOOR_RETURN',
    256: 'doorfs(2) - DOOR_CREATE',
    257: 'doorfs(2) - DOOR_REVOKE',
    258: 'doorfs(2) - DOOR_INFO',
    259: 'doorfs(2) - DOOR_CRED',
    260: 'doorfs(2) - DOOR_BIND',
    261: 'doorfs(2) - DOOR_UNBIND',
    262: 'p_online(2)',
    263: 'processor_bind(2)',
    264: 'inst_sync(2)',
    265: 'configure socket',
    266: 'setaudit_addr(2)',
    267: 'getaudit_addr(2)',
    268: 'Solaris umount(2)',
    269: 'fsat(2) - place holder',
    270: 'openat(2) - read',
    271: 'openat(2) - read,creat',
    272: 'openat(2) - read,trunc',
    273: 'openat(2) - read,creat,trunc',
    274: 'openat(2) - write',
    275: 'openat(2) - write,creat',
    276: 'openat(2) - write,trunc',
    277: 'openat(2) - write,creat,trunc',
    278: 'openat(2) - read,write',
    279: 'openat(2) - read,write,create',
    280: 'openat(2) - read,write,trunc',
    281: 'openat(2) - read,write,creat,trunc',
    282: 'renameat(2)',
    283: 'fstatat(2)',
    284: 'fchownat(2)',
    285: 'futimesat(2)',
    286: 'unlinkat(2)',
    287: 'clock_settime(2)',
    288: 'ntp_adjtime(2)',
    289: 'setppriv(2)',
    290: 'modctl(2) - configure device policy',
    291: 'modctl(2) - configure additional privilege',
    292: 'kernel cryptographic framework',
    293: 'configure kernel SSL',
    294: 'brandsys(2)',
    295: 'Add IPsec policy rule',
    296: 'Delete IPsec policy rule',
    297: 'Clone IPsec policy',
    298: 'Flip IPsec policy',
    299: 'Flush IPsec policy rules',
    300: 'Update IPsec algorithms',
    301: 'portfs',
    302: 'ptrace(2)',
    303: 'chflags(2)',
    304: 'fchflags(2)',
    305: 'profil(2)',
    306: 'ktrace(2)',
    307: 'setlogin(2)',
    308: 'reboot(2)',
    309: 'revoke(2)',
    310: 'umask(2)',
    311: 'mprotect(2)',
    312: 'setpriority(2)',
    313: 'settimeofday(2)',
    314: 'flock(2)',
    315: 'mkfifo(2)',
    316: 'poll(2)',
    317: 'socketpair(2)',
    318: 'futimes(2)',
    319: 'setsid(2)',
    320: 'setprivexec(2)',
    321: 'nfssvc(2)',
    322: 'getfh(2)',
    323: 'quotactl(2)',
    324: 'add_profil()',
    325: 'kdebug_trace()',
    326: 'fstat(2)',
    327: 'fpathconf(2)',
    328: 'getdirentries(2)',
    329: 'truncate(2)',
    330: 'ftruncate(2)',
    331: 'sysctl(3)',
    332: 'mlock(2)',
    333: 'munlock(2)',
    334: 'undelete(2)',
    335: 'getattrlist()',
    336: 'setattrlist()',
    337: 'getdirentriesattr()',
    338: 'exchangedata()',
    339: 'searchfs()',
    340: 'minherit(2)',
    341: 'semconfig()',
    342: 'sem_open(2)',
    343: 'sem_close(2)',
    344: 'sem_unlink(2)',
    345: 'shm_open(2)',
    346: 'shm_unlink(2)',
    347: 'load_shared_file()',
    348: 'reset_shared_file()',
    349: 'new_system_share_regions()',
    350: 'pthread_kill(2)',
    351: 'pthread_sigmask(2)',
    352: 'auditctl(2)',
    353: 'rfork(2)',
    354: 'lchmod(2)',
    355: 'swapoff(2)',
    356: 'init_process()',
    357: 'map_fd()',
    358: 'task_for_pid()',
    359: 'pid_for_task()',
    360: 'sysctl() - non-admin',
    361: 'copyfile()',
    43001: 'getfsstat(2)',
    43002: 'ptrace(2)',
    43003: 'chflags(2)',
    43004: 'fchflags(2)',
    43005: 'profil(2)',
    43006: 'ktrace(2)',
    43007: 'setlogin(2)',
    43008: 'revoke(2)',
    43009: 'umask(2)',
    43010: 'mprotect(2)',
    43011: 'mkfifo(2)',
    43012: 'poll(2)',
    43013: 'futimes(2)',
    43014: 'setsid(2)',
    43015: 'setprivexec(2)',
    43016: 'add_profil()',
    43017: 'kdebug_trace()',
    43018: 'fstat(2)',
    43019: 'fpathconf(2)',
    43020: 'getdirentries(2)',
    43021: 'sysctl(3)',
    43022: 'mlock(2)',
    43023: 'munlock(2)',
    43024: 'undelete(2)',
    43025: 'getattrlist()',
    43026: 'setattrlist()',
    43027: 'getdirentriesattr()',
    43028: 'exchangedata()',
    43029: 'searchfs()',
    43030: 'minherit(2)',
    43031: 'semconfig()',
    43032: 'sem_open(2)',
    43033: 'sem_close(2)',
    43034: 'sem_unlink(2)',
    43035: 'shm_open(2)',
    43036: 'shm_unlink(2)',
    43037: 'load_shared_file()',
    43038: 'reset_shared_file()',
    43039: 'new_system_share_regions()',
    43040: 'pthread_kill(2)',
    43041: 'pthread_sigmask(2)',
    43042: 'auditctl(2)',
    43043: 'rfork(2)',
    43044: 'lchmod(2)',
    43045: 'swapoff(2)',
    43046: 'init_process()',
    43047: 'map_fd()',
    43048: 'task_for_pid()',
    43049: 'pid_for_task()',
    43050: 'sysctl() - non-admin',
    43051: 'copyfile(2)',
    43052: 'lutimes(2)',
    43053: 'lchflags(2)',
    43054: 'sendfile(2)',
    43055: 'uselib(2)',
    43056: 'getresuid(2)',
    43057: 'setresuid(2)',
    43058: 'getresgid(2)',
    43059: 'setresgid(2)',
    43060: 'wait4(2)',
    43061: 'lgetfh(2)',
    43062: 'fhstatfs(2)',
    43063: 'fhopen(2)',
    43064: 'fhstat(2)',
    43065: 'jail(2)',
    43066: 'eaccess(2)',
    43067: 'kqueue(2)',
    43068: 'kevent(2)',
    43069: 'fsync(2)',
    43070: 'nmount(2)',
    43071: 'bdflush(2)',
    43072: 'setfsuid(2)',
    43073: 'setfsgid(2)',
    43074: 'personality(2)',
    43075: 'getscheduler(2)',
    43076: 'setscheduler(2)',
    43077: 'prctl(2)',
    43078: 'getcwd(2)',
    43079: 'capget(2)',
    43080: 'capset(2)',
    43081: 'pivot_root(2)',
    43082: 'rtprio(2)',
    43083: 'sched_getparam(2)',
    43084: 'sched_setparam(2)',
    43085: 'sched_get_priority_max(2)',
    43086: 'sched_get_priority_min(2)',
    43087: 'sched_rr_get_interval(2)',
    43088: 'acl_get_file(2)',
    43089: 'acl_set_file(2)',
    43090: 'acl_get_fd(2)',
    43091: 'acl_set_fd(2)',
    43092: 'acl_delete_file(2)',
    43093: 'acl_delete_fd(2)',
    43094: 'acl_aclcheck_file(2)',
    43095: 'acl_aclcheck_fd(2)',
    43096: 'acl_get_link(2)',
    43097: 'acl_set_link(2)',
    43098: 'acl_delete_link(2)',
    43099: 'acl_aclcheck_link(2)',
    43100: 'sysarch(2)',
    43101: 'extattrctl(2)',
    43102: 'extattr_get_file(2)',
    43103: 'extattr_set_file(2)',
    43104: 'extattr_list_file(2)',
    43105: 'extattr_delete_file(2)',
    43106: 'extattr_get_fd(2)',
    43107: 'extattr_set_fd(2)',
    43108: 'extattr_list_fd(2)',
    43109: 'extattr_delete_fd(2)',
    43110: 'extattr_get_link(2)',
    43111: 'extattr_set_link(2)',
    43112: 'extattr_list_link(2)',
    43113: 'extattr_delete_link(2)',
    43114: 'kenv(8)',
    43115: 'jail_attach(2)',
    43116: 'sysctl(3)',
    43117: 'linux ioperm',
    43118: 'readdir(3)',
    43119: 'linux iopl',
    43120: 'linux vm86',
    43121: 'mac_get_proc(2)',
    43122: 'mac_set_proc(2)',
    43123: 'mac_get_fd(2)',
    43124: 'mac_get_file(2)',
    43125: 'mac_set_fd(2)',
    43126: 'mac_set_file(2)',
    43127: 'mac_syscall(2)',
    43128: 'mac_get_pid(2)',
    43129: 'mac_get_link(2)',
    43130: 'mac_set_link(2)',
    43131: 'mac_execve(2)',
    43132: 'getpath_fromfd(2)',
    43133: 'getpath_fromaddr(2)',
    43134: 'mq_open(2)',
    43135: 'mq_setattr(2)',
    43136: 'mq_timedreceive(2)',
    43137: 'mq_timedsend(2)',
    43138: 'mq_notify(2)',
    43139: 'mq_unlink(2)',
    43140: 'listen(2)',
    43141: 'mlockall(2)',
    43142: 'munlockall(2)',
    43143: 'closefrom(2)',
    43144: 'fexecve(2)',
    43145: 'faccessat(2)',
    43146: 'fchmodat(2)',
    43147: 'linkat(2)',
    43148: 'mkdirat(2)',
    43149: 'mkfifoat(2)',
    43150: 'mknodat(2)',
    43151: 'readlinkat(2)',
    43152: 'symlinkat(2)',
    43153: 'mac_getfsstat(2)',
    43154: 'mac_get_mount(2)',
    43155: 'mac_get_lcid(2)',
    43156: 'mac_get_lctx(2)',
    43157: 'mac_set_lctx(2)',
    43158: 'mac_mount(2)',
    43159: 'getlcid(2)',
    43160: 'setlcid(2)',
    43161: 'taskname_for_pid()',
    43162: 'access_extended(2)',
    43163: 'chmod_extended(2)',
    43164: 'fchmod_extended(2)',
    43165: 'fstat_extended(2)',
    43166: 'lstat_extended(2)',
    43167: 'mkdir_extended(2)',
    43168: 'mkfifo_extended(2)',
    43169: 'open_extended(2) - attr only',
    43170: 'open_extended(2) - read',
    43171: 'open_extended(2) - read,creat',
    43172: 'open_extended(2) - read,trunc',
    43173: 'open_extended(2) - read,creat,trunc',
    43174: 'open_extended(2) - write',
    43175: 'open_extended(2) - write,creat',
    43176: 'open_extended(2) - write,trunc',
    43177: 'open_extended(2) - write,creat,trunc',
    43178: 'open_extended(2) - read,write',
    43179: 'open_extended(2) - read,write,creat',
    43180: 'open_extended(2) - read,write,trunc',
    43181: 'open_extended(2) - read,write,creat,trunc',
    43182: 'stat_extended(2)',
    43183: 'umask_extended(2)',
    43184: 'openat(2) - attr only',
    43185: 'posix_openpt(2)',
    43186: 'cap_new(2)',
    43187: 'cap_getrights(2)',
    43188: 'cap_enter(2)',
    43189: 'cap_getmode(2)',
    43190: 'posix_spawn(2)',
    43191: 'fsgetpath(2)',
    43192: 'pread(2)',
    43193: 'pwrite(2)',
    43194: 'fsctl()',
    43195: 'ffsctl()',
    43196: 'lpathconf(2)',
    43197: 'pdfork(2)',
    43198: 'pdkill(2)',
    43199: 'pdgetpid(2)',
    43200: 'pdwait(2)',
    44901: 'session start',
    44902: 'session update',
    44903: 'session end',
    44904: 'session close',
    6144: 'at-create atjob',
    6145: 'at-delete atjob (at or atrm)',
    6146: 'at-permission',
    6147: 'cron-invoke',
    6148: 'crontab-crontab created',
    6149: 'crontab-crontab deleted',
    6150: 'crontab-permission',
    6151: 'inetd connection',
    6152: 'login - local',
    6153: 'logout - local',
    6154: 'login - telnet',
    6155: 'login - rlogin',
    6156: 'mount',
    6157: 'unmount',
    6158: 'rsh access',
    6159: 'su(1)',
    6160: 'system halt',
    6161: 'system reboot',
    6162: 'rexecd',
    6163: 'passwd',
    6164: 'rexd',
    6165: 'ftp access',
    6166: 'init',
    6167: 'uadmin',
    6168: 'system shutdown',
    6170: 'crontab-modify',
    6171: 'ftp logout',
    6172: 'login - ssh',
    6173: 'role login',
    6180: ' profile command',
    6181: 'add filesystem',
    6182: 'delete filesystem',
    6183: 'modify filesystem',
    6200: 'allocate-device success',
    6201: 'allocate-device failure',
    6202: 'deallocate-device success',
    6203: 'deallocate-device failure',
    6204: 'allocate-list devices success',
    6205: 'allocate-list devices failure',
    6207: 'create user',
    6208: 'modify user',
    6209: 'delete user',
    6210: 'disable user',
    6211: 'enable user',
    6212: 'newgrp login',
    6213: 'admin login',
    6214: 'authenticated kadmind request',
    6215: 'unauthenticated kadmind req',
    6216: 'kdc authentication svc request',
    6217: 'kdc tkt-grant svc request',
    6218: 'kdc tgs 2ndtkt mismtch',
    6219: 'kdc tgs issue alt tgt',
    6300: 'sudo(1)',
    6501: 'modify password',
    6511: 'create group',
    6512: 'delete group',
    6513: 'modify group',
    6514: 'add to group',
    6515: 'remove from group',
    6521: 'revoke object priv',
    6600: 'loginwindow login',
    6601: 'loginwindow logout',
    7000: 'user authentication',
    7001: 'SecSrvr connection setup',
    7002: 'SecSrvr AuthEngine',
    7003: 'SecSrvr authinternal mech',
    32800: 'OpenSSH login',
    45000: 'audit startup',
    45001: 'audit shutdown',
    45014: 'modify password',
    45015: 'create group',
    45016: 'delete group',
    45017: 'modify group',
    45018: 'add to group',
    45019: 'remove from group',
    45020: 'revoke object priv',
    45021: 'loginwindow login',
    45022: 'loginwindow logout',
    45023: 'user authentication',
    45024: 'SecSrvr connection setup',
    45025: 'SecSrvr AuthEngine',
    45026: 'SecSrvr authinternal mech',
    45027: 'Calife',
    45028: 'sudo(1)',
    45029: 'audit crash recovery',
    45030: 'SecSrvr AuthMechanism',
    45031: 'Security Assessment'
}
